//Chris Eng & Ken Rice
//CSS450 Sung

// Blob.cpp
#include "stdafx.h"
#include "Blob.h"
#include "EscapeProject.h"

CBlob::CBlob(const wchar_t* name) : UWB_SceneNode(name) {

	m_pBodyComponents = new UWB_PrimitiveList();

	//The body
	UWB_PrimitiveRectangleXY* pBody = new UWB_PrimitiveRectangleXY();
	pBody->SetCorners( vec3(-7,-7,0), vec3(7,7,0) );
	pBody->SetFlatColor( UWB_Color( 50,50,255));
	pBody->SetShadeMode( smFlat );
	pBody->EnableTexturing(true);
	pBody->EnableBlending(true);
	wchar_t* wstr_File = L"Texture/Amoeba.png";
	pBody->SetTextureFileName(wstr_File);
	m_pBodyComponents->Append( pBody );

	//Attatch the body components as the primitive of a body SceneNode
	UWB_SceneNode* pBodyNode = new UWB_SceneNode(L"Body");
	pBodyNode->SetPrimitive(m_pBodyComponents);

	//Attatch all SceneNodes to the Blob SceneNode
	InsertChildNode(pBodyNode);		//GetChildNode( 0 )

	m_timer = 0.0f;
	m_state = bWANDERING;
	m_stunTimer = 0;
	m_rotation = 0.0f;
	m_textureState = 1;
	m_stateDirectionDown = false;
	m_direction = GO_UP;
	m_moveTimer = 0;

	m_leftWallCnt = 0;
	m_rightWallCnt = 0;
	m_downWallCnt = 0;
	m_upWallCnt = 0;
}

CBlob::~CBlob() {
	
	//Must nullify these pointers to avoid a crash!
	//m_pBodyComponents = NULL;

	//Now call the superclass to clean up.
	//UWB_SceneNode::~UWB_SceneNode();
}

void CBlob::DoMove(const UWB_BoundingVolume* targetBound, vec3 itemCenter, float difficulty) {

	m_difficulty = difficulty;

	m_stunTimer--;
	if(m_stunTimer > 0)
		return;

	int per = (int)UWB_RandomNumber(1, 100);
	vec3 ghostCenter = GetCenterInWC();
	vec3 heroCenter = targetBound->GetCenter();
	float distance = GetDistance(ghostCenter, heroCenter);

	if(distance < 10)
		DoHunting(targetBound);
	else
	{
		if(per <= 60)
			DoWandering();
		else
			DoHunting(targetBound);
	}

	DoStateMachine(targetBound, itemCenter);
}

void CBlob::DoWandering() {

	
	if(m_moveTimer >= 20) // .5 * 40 times a second
	{
		m_direction = (int)UWB_RandomNumber(1, 4);
		m_moveTimer = 0.0f;
	}
	
	if(m_moveTimer == 0.0f)
	{
		if(m_direction == GO_UP)
			m_rotation = UWB_PI / 2.0f;
		if(m_direction == GO_DOWN)
			m_rotation = (3.0f * UWB_PI) / 2.0f;
		if(m_direction == GO_LEFT)
			m_rotation = UWB_PI;
		if(m_direction == GO_RIGHT)
			m_rotation = 0.0f;

		// Go in the oppostie direction when "stuck"
		// Decide not to use
		/*if(m_leftWallCnt >= 40)
		{
			m_leftWallCnt = 0;
			m_rotation = 0.0f;
		}
		if(m_rightWallCnt >= 40)
		{
			m_rightWallCnt = 0;
			m_rotation = UWB_PI;
		}
		if(m_downWallCnt >= 40)
		{
			m_downWallCnt = 0;
			m_rotation = UWB_PI / 2.0f;
		}
		if(m_upWallCnt >= 40)
		{
			m_upWallCnt = 0;
			m_rotation = (3.0f * UWB_PI) / 2.0f;
		}*/
	}

	WalkForwards(m_rotation, 1.5f);
	m_moveTimer++;
}

vec3 CBlob::CheckHeroCollision(const UWB_BoundingVolume* otherBound, const vec3 otherTranslation) {

	vec3 newTranslation = otherTranslation;

	UWB_PrimitiveList* pBlobComponents = static_cast<UWB_PrimitiveList*>(GetChildNode(0)->GetPrimitive());
	UWB_Primitive* pBlob = pBlobComponents->PrimitiveAt(0);
	const UWB_BoundingBox* pBlobBounds = static_cast<const UWB_BoundingBox*>(pBlob->GetBoundingVolume(theApp.GetModel().GetDrawHelper()->GetLOD() ) );

	vec3 corner1 = pBlobBounds->GetMin();
	vec3 corner2 = pBlobBounds->GetMax();
	UWBD3D_DrawHelper* drawHelper = theApp.GetModel().GetDrawHelper();
	drawHelper->PushModelTransform();
	GetXformInfo().SetUpModelStack(*drawHelper);
	drawHelper->TransformPoint(corner1);
	drawHelper->TransformPoint(corner2);
	drawHelper->PopModelTransform();
	const UWB_BoundingBox blobTransformed = UWB_BoundingBox(corner1, corner2);

	if(otherBound->Intersects(blobTransformed)) {
		// Downgrade hero's health
		//theApp.GetModel().HandleHealth(); -- done in DoStateMachine now

		// move hero?
		//newTranslation.x = ?;
		//newTranslation.y = ?;

		// move blob?
		//vec3 translation = m_xform.GetTranslation();
		//m_xform.SetTranslation(vec3(translation.x + ?, translation.y + ?, 0.0f));
	}

	//Return a translation, which may or may not have been readjusted
	return newTranslation;
}

void CBlob::WalkForwards(float rotation, float speed) {
	
	float xComponent = cos(rotation) * speed * m_difficulty;
	float yComponent = sin(rotation) * speed * m_difficulty;

	vec3 translation = m_xform.GetTranslation();
	m_xform.SetTranslation(vec3(translation.x + xComponent, translation.y + yComponent, 0.0f));
}

void CBlob::DoHunting(const UWB_BoundingVolume* targetBound) {
	
	vec3 targetCenter = targetBound->GetCenter();
	vec3 ghostCenter = GetCenterInWC();

	float dx = targetCenter.x - ghostCenter.x;
	float dy = targetCenter.y - ghostCenter.y;
	float targetAngle = atan2(dy, dx);
	WalkForwards(targetAngle, 0.6f);
}

float CBlob::GetDistance(vec3 start, vec3 end)
{
	float dx = (float)start.x - (float)end.x;
	float dy = (float)start.y - (float)end.y;
	float dz = (float)start.z - (float)end.z;
	float length = sqrtf(dx*dx + dy*dy + dz*dz);
	return length;
}

vec3 CBlob::GetCenterInWC() {

	UWB_PrimitiveList* pBlobComponents = static_cast<UWB_PrimitiveList*>(GetChildNode(0)->GetPrimitive());
	UWB_Primitive* pBlob = pBlobComponents->PrimitiveAt(0);
	const UWB_BoundingBox* pBlobBounds = static_cast<const UWB_BoundingBox*>(pBlob->GetBoundingVolume(theApp.GetModel().GetDrawHelper()->GetLOD() ) );
	vec3 corner1 = pBlobBounds->GetMin();
	vec3 corner2 = pBlobBounds->GetMax();
	UWBD3D_DrawHelper* drawHelper = theApp.GetModel().GetDrawHelper();
	drawHelper->PushModelTransform();
	GetXformInfo().SetUpModelStack(*drawHelper);
	drawHelper->TransformPoint(corner1);
	drawHelper->TransformPoint(corner2);
	drawHelper->PopModelTransform();
	const UWB_BoundingBox blobTransformed = UWB_BoundingBox(corner1, corner2);
	vec3 blobCenter = blobTransformed.GetCenter();
	return blobCenter;
}

void CBlob::DoGuarding(vec3 itemCenter) {

	m_xform.SetTranslation(itemCenter);
}

void CBlob::DoReturning(vec3 itemCenter) {

}

void CBlob::DoStateMachine(const UWB_BoundingVolume* targetBound, vec3 itemCenter) {

	vec3 ghostCenter = GetCenterInWC();
	vec3 heroCenter = targetBound->GetCenter();
	float distance = GetDistance(ghostCenter, heroCenter);

	if(distance < 8) // do instead of "heroBound->Intersects(blobTransformedBoundingBox)"
		theApp.GetModel().HandleHealth();

	if(m_timer >= 10)
	{
		if(m_textureState < 1)
		{
			m_textureState = 1;
			m_stateDirectionDown = true;
		}
		else if(m_textureState > 5)
		{
			m_textureState = 5;
			m_stateDirectionDown = false;
		}
		else if(m_textureState == 1)
			m_stateDirectionDown = false;
		else if(m_textureState == 5)
			m_stateDirectionDown = true;

		if(m_stateDirectionDown)
			m_textureState--;
		else
			m_textureState++;

		AlternateTexture(m_textureState);
		m_timer = 0.0f;	
	}
	m_timer++;
}

void CBlob::AlternateTexture(int index) {

	UWB_PrimitiveList* pBlobComponents = static_cast<UWB_PrimitiveList*>(GetChildNode(0)->GetPrimitive());
	UWB_Primitive* pBlob = pBlobComponents->PrimitiveAt(0);

	wchar_t* wstr_File;
	if(index == 1)
		wstr_File = L"Texture/AmoebaSmall1.png";
	if(index == 2)
		wstr_File = L"Texture/AmoebaSmall2.png";
	if(index == 3)
		wstr_File = L"Texture/Amoeba.png";
	if(index == 4)
		wstr_File = L"Texture/AmoebaBig1.png";
	if(index == 5)
		wstr_File = L"Texture/AmoebaBig2.png";

	pBlob->SetTextureFileName(wstr_File);
}

int CBlob::GetStunned() const {
	//return (m_stunTimer > 0);
	return m_stunTimer;
}

void CBlob::SetStunned(int stunTimer) {
	m_stunTimer = stunTimer;
}

// not use currently but can get which wall hit
// then use to improve the blob's "random" movement away from highly hit walls
void CBlob::SetWhichWallHit(int wall) {

	if(wall == 4)
		m_leftWallCnt++;
	if(wall == 3)
		m_rightWallCnt++;
	if(wall == 2)
		m_downWallCnt++;
	if(wall == 1)
		m_upWallCnt++;
}
