//Chris Eng & Ken Rice

#include "stdafx.h"
#include "CDarknessGrid.h"
#include "EscapeProject.h"

//Default constructor does nothing!
CDarknessGrid::CDarknessGrid(const wchar_t* name) {
	SetName(name);
}

void CDarknessGrid::Initialize(int rows, int cols, float levelWidth, float levelHeight, float levelScale, float fadeRate) {
	m_Rows = rows;
	m_Cols = cols;
	m_LevelScale = levelScale;
	m_TileHeight = levelHeight / m_Rows;
	m_TileWidth = levelWidth / m_Cols;

	//test
	m_TileLife = new float[cols * rows];
	//Must pretend that a 1D array is 2D to allow dynamic allocation, or give up dynamic variables for 2D-ness
	m_DarknessTiles = new UWB_PrimitiveRectangleXY[cols * rows];
	for(int r = 0; r < m_Rows; r++) {
		for(int c = 0; c < m_Cols; c++) {

			vec3 corner1(r * m_TileWidth, c * m_TileHeight, 0.0f);
			vec3 corner2 = corner1 + vec3(m_TileWidth, m_TileHeight, 0.0f);

			UWB_PrimitiveRectangleXY currentRect = UWB_PrimitiveRectangleXY();
			currentRect.SetCorners(corner1, corner2);
			currentRect.SetShadeMode(smFlat);
			currentRect.EnableBlending(true); //was false
			currentRect.EnableTexturing(false);

			m_DarknessTiles[c * m_Cols + r] = currentRect;
			m_TileLife[c * m_Cols + r] = 0.0f;
		}
	}

	//Only make the darkness frame if it hasn't been made yet (possibly restarting the game...)
	if(GetChildNode(0) == NULL) {
		UWB_PrimitiveList* pShroudTypes = new UWB_PrimitiveList();

		//List 0: cell phone -- 4 layers of primtive circles 
		UWB_PrimitiveList* pCellPhoneFrame = new UWB_PrimitiveList();
		for(int i = 0; i < 4; i++) {
			MakeShroudFrame(vec3(0.0f, 0.0f, 0.0f), 12.75f, NULL, UWB_Color(0.0f, 0.0f, 0.0f, 1.0f), pCellPhoneFrame);		
		}
		pShroudTypes->Append(pCellPhoneFrame);

		//List 1: flashlight normal -- 1 narrow wedge layered above 1 wider wedge above 1 primitive circle
		UWB_PrimitiveList* pFLNormalFrame = new UWB_PrimitiveList();
		MakeShroudFrame(vec3(0.0f, 0.0f, 0.0f), 12.75f, NULL, UWB_Color(0.0f, 0.0f, 0.0f, 1.0f), pFLNormalFrame);		
		MakeShroudFrame(vec3(0.0f, 0.0f, 0.0f), 12.75f, UWB_PI / 4, UWB_Color(0.0f, 0.0f, 0.0f, 1.0f), pFLNormalFrame);
		MakeShroudFrame(vec3(0.0f, 0.0f, 0.0f), 12.75f, UWB_PI / 8, UWB_Color(0.0f, 0.0f, 0.0f, 1.0f), pFLNormalFrame);
		pShroudTypes->Append(pFLNormalFrame);

		//List 2: flashlight high -- 1 wide wedge layered above 1 wider wedge
		UWB_PrimitiveList* pFLHighFrame = new UWB_PrimitiveList();
		MakeShroudFrame(vec3(0.0f, 0.0f, 0.0f), 12.75f, UWB_PI / 2, UWB_Color(0.0f, 0.0f, 0.0f, 1.0f), pFLHighFrame);
		MakeShroudFrame(vec3(0.0f, 0.0f, 0.0f), 12.75f, UWB_PI / 4, UWB_Color(0.0f, 0.0f, 0.0f, 1.0f), pFLHighFrame);
		pShroudTypes->Append(pFLHighFrame);

		//List four: generic mouse pan -- ???

		UWB_SceneNode* m_DarknessFrame = new UWB_SceneNode(L"Shroud Frame");
		m_DarknessFrame->SetPrimitive(pShroudTypes);
		InsertChildNode(m_DarknessFrame);

		m_TileFadeRate = 0.01f;
		SetTileFadeRate(fadeRate);
	}
}

//Used for making both wedge AND circle shroud frames!
void CDarknessGrid::MakeShroudFrame(vec3 center, float radius, float wedgeRadians, UWB_Color edgeColor, UWB_PrimitiveList* gradientCollection) {
	
	if(wedgeRadians == NULL) {
		UWB_PrimitiveCircle* pCircle = new UWB_PrimitiveCircle();
		pCircle->SetCenter(0.0f, 0.0f, 0.0f);
		pCircle->SetFlatColor(edgeColor);
		pCircle->SetRadius(radius);
		pCircle->EnableBlending(true);
		pCircle->SetShadeMode( smGouraud );
		pCircle->SetShadingColor(UWB_Color(0.0f, 0.0f, 0.0f, 0.0f));

		gradientCollection->Append(pCircle);

	} else { 
		CWedge* pWedge = new CWedge(wedgeRadians);
		pWedge->SetCenter(0.0f, 0.0f, 0.0f);
		pWedge->SetFlatColor(edgeColor);
		pWedge->SetRadius(radius);
		pWedge->EnableBlending(true);
		pWedge->SetShadeMode( smGouraud );
		pWedge->SetShadingColor(UWB_Color(0.0f, 0.0f, 0.0f, 0.0f));

		gradientCollection->Append(pWedge);
	}
}

void CDarknessGrid::RevealTile(const UWB_BoundingVolume* otherBound) {

	vec3 otherTile = otherBound->GetCenter();
	otherTile.x = otherTile.x / (m_TileWidth * m_LevelScale);
	otherTile.y = otherTile.y / (m_TileHeight * m_LevelScale);

	//Check the tiles immediately surrounding the hero (5x5)
	for(int r = (int)otherTile.x - GOOD_TILE_RANGE; r <= otherTile.x + GOOD_TILE_RANGE; r++) {
		for(int c = (int)otherTile.y - GOOD_TILE_RANGE; c <= otherTile.y + GOOD_TILE_RANGE; c++) {
			
			//Check that this [c][r] is valid
			if(-1 < r && r < m_Rows && -1 < c && c < m_Cols) {
				
				//make visible tiles within the hero's range invisible
				if(m_DarknessTiles[c * m_Cols + r].IsVisible()) {
					m_DarknessTiles[c * m_Cols + r].SetFlatColor(UWB_Color(0.0f, 0.0f, 0.0f, 0.0f));
					m_TileLife[c * m_Cols + r] = 1.0f;
				}
			}
		}
	}
}

void CDarknessGrid::TransformShroudFrame(const vec3 newTranslation, const float newRotationRadians) {
	UWB_XformInfo frameXform = GetChildNode(0)->GetXformInfo();
	
	if(newTranslation != NULL) {
		frameXform.SetTranslation(vec3(newTranslation.x / theApp.GetModel().GetCurrentLevelScale().x/*LEVEL_ONE_SCALE*/, newTranslation.y / theApp.GetModel().GetCurrentLevelScale().y/*LEVEL_ONE_SCALE*/, 0.0f)); //the many ways to pass in newTranslation already has level one scale multiplied in, appearantly need it without
	}
	
	if(newRotationRadians != NULL) {
		frameXform.SetRotationRadians(newRotationRadians);
	}

	GetChildNode(0)->SetXformInfo(frameXform);
}

int CDarknessGrid::GetDarknessLevel() {
	int level = 0;

	UWB_PrimitiveList* pShroudTypes = static_cast<UWB_PrimitiveList*>(GetChildNode(0)->GetPrimitive());
	for(int i = 0; i < pShroudTypes->Count(); i++) {

		if(pShroudTypes->PrimitiveAt(i)->IsVisible()) {
			level = i;
		}
	}
	return level;

	/*int level = 0;

	UWB_PrimitiveList* pDarknessComponents = static_cast<UWB_PrimitiveList*>(GetChildNode(0)->GetPrimitive());

	for(int i = 0; i < pDarknessComponents->Count(); i++) {
		UWB_Primitive* pCurrentFrame = pDarknessComponents->PrimitiveAt(i);
		
		if(pCurrentFrame->IsVisible() == false) {
			level++;
		}
	}*/

	return level;
}

//0: cellphone level 1: flashlight normal level 2: flashlight high level
void CDarknessGrid::SetDarknessLevel(int level) {

	UWB_PrimitiveList* pShroudList = static_cast<UWB_PrimitiveList*>(GetChildNode(0)->GetPrimitive());
	
	//Just turn off all shrouds, then turn on the requested one
	for(int i = 0; i < pShroudList->Count(); i++) {
		pShroudList->PrimitiveAt(i)->SetVisible(false);

		if(i == 0 && level == 0) {
			pShroudList->PrimitiveAt(i)->SetVisible(true);
		} else if (i == 1 && level == 1) {
			pShroudList->PrimitiveAt(i)->SetVisible(true);
		} else if (i == 2 && level == 2) {
			pShroudList->PrimitiveAt(i)->SetVisible(true);
		}
	}
}

void CDarknessGrid::SetTileFadeRate(float newRate) {
	if(newRate >= 0.0f) {
		m_TileFadeRate = newRate;
	}
}

//Specialized draw method since using array to rather than PrimitiveList
void CDarknessGrid::Draw( eLevelOfDetail lod, UWB_DrawHelper& draw_helper, bool drawDarkFrame ) const {
	draw_helper.PushModelTransform();
	m_xform.SetUpModelStack( draw_helper );

	//Draw the darkness grid
	for(int r = 0; r < m_Rows; r++) {
		for(int c = 0; c < m_Cols; c++) {

			if(-UWB_ZERO_TOLERANCE <= m_TileLife[c * m_Cols + r] && m_TileLife[c * m_Cols + r] <= UWB_ZERO_TOLERANCE) {
				m_TileLife[c * m_Cols + r] = 0.0f;
			} else {
				m_TileLife[c * m_Cols + r] -= m_TileFadeRate;
			}

			UWB_Color newTileColor(0.0f, 0.0f, 0.0f, 1.0f - m_TileLife[c * m_Cols + r]); //= m_DarknessTiles[c * m_Cols + r].GetFlatColor();
			m_DarknessTiles[c * m_Cols + r].SetFlatColor(newTileColor);

			m_DarknessTiles[c * m_Cols + r].Draw(lod, draw_helper);
		}	
	}
	draw_helper.PopModelTransform();

	//This call will draw the darkness following the hero
	if(drawDarkFrame) {
		UWB_SceneNode::Draw(lod, draw_helper);
	}
}
