//Chris Eng & Ken Rice
//CSS450 Sung

// Level.cpp

#include "stdafx.h"
#include "CLevel.h"
#include "EscapeProject.h"

//No Initialize() superfunction here!
void CLevel::Initialize() {};
int CLevel::GetLevelNumber() const { return -1; }

CLevel::~CLevel() {
	
	//Must nullify these pointers to avoid a crash!
	pWallComponents = NULL;
	pDoorComponents = NULL;
	pItemComponents = NULL;

	//Now call the superclass to clean up.
	UWB_SceneNode::~UWB_SceneNode();
}

UWB_BoundingBox CLevel::GetLevelDimension() {
	return UWB_BoundingBox(vec3(0.0f, 0.0f, 0.0f), (float)m_WorldWidth, (float)m_WorldHeight, 0.0f);
}

vec3 CLevel::GetLevelShroudTilesDimensions() {
	return vec3((float)m_TileCols, (float)m_TileRows, 0.0f);
}

UWB_Color CLevel::GetLevelShroudColor() {
	return m_ShroudColor;
}

UWB_XformInfo CLevel::GetLevelStart() {
	return m_Start_Level;
}

UWB_XformInfo CLevel::GetHeroStart() {
	return m_Start_Hero;
}

UWB_XformInfo CLevel::GetGhostStart() {
	return m_Start_Ghost;
}

UWB_XformInfo CLevel::GetBlobStart() {
	return m_Start_Blob;
}

UWB_XformInfo CLevel::GetZombieStart() {
	return m_Start_Zombie;
}

vec3 CLevel::CheckWallCollision(const UWB_BoundingVolume* otherBound, const vec3 otherTranslation) {

	int wall;
	return CheckWallCollision(otherBound, otherTranslation, wall);
}

vec3 CLevel::CheckWallCollision(const UWB_BoundingVolume* otherBound, const vec3 otherTranslation, int& wall) {
	vec3 newTranslation = otherTranslation;
	
	if(otherBound->GetType() == UWB_BoundingVolume::box) {
	
		vec3 boundCenter = otherBound->GetCenter();
		float boundWidth = static_cast<const UWB_BoundingBox*>(otherBound)->Width();
		float boundHeight = static_cast<const UWB_BoundingBox*>(otherBound)->Height();

		vec3 heroTile = boundCenter;
		heroTile.x = heroTile.x / (m_TileWidth * m_Start_Level.GetScale().x/*LEVEL_ONE_SCALE*/);
		heroTile.y = heroTile.y / (m_TileHeight * m_Start_Level.GetScale().y/*LEVEL_ONE_SCALE*/);

		//Check for a wall on the tile to the left
		if(m_TileArray[int(heroTile.x - 1) * m_TileCols + int(heroTile.y)] == WALL) {
			float leftEnd = float(int(heroTile.x) * m_TileWidth * m_Start_Level.GetScale().x/*LEVEL_ONE_SCALE*/);
			float boundExtreme = boundCenter.x - boundWidth / 2;
			float overlap = leftEnd - boundExtreme;
			if(overlap > 0) {
				newTranslation.x = newTranslation.x + overlap;
				wall = 4;
			}
		}
		//Check for a wall on the tile to the right
		if(m_TileArray[int(heroTile.x + 1) * m_TileCols + int(heroTile.y)] == WALL) {
			float rightEnd = float(int(heroTile.x + 1) * m_TileWidth * m_Start_Level.GetScale().x/*LEVEL_ONE_SCALE*/);
			float boundExtreme = boundCenter.x + boundWidth / 2;
			float overlap = rightEnd - boundExtreme;
			if(overlap < 0) {
				newTranslation.x = newTranslation.x + overlap; //overlap is negative
				wall = 3;
			}
		}
		//Check for a wall on the tile to the south
		if(m_TileArray[int(heroTile.x) * m_TileCols + int(heroTile.y - 1)] == WALL) {
			float southEnd = float(int(heroTile.y) * m_TileHeight * m_Start_Level.GetScale().y/*LEVEL_ONE_SCALE*/);
			float boundExtreme = boundCenter.y - boundHeight / 2;
			float overlap = southEnd - boundExtreme;
			if(overlap > 0) {
				newTranslation.y = newTranslation.y + overlap;
				wall = 2;
			}
		}
		//Check for a wall on the tile to the north
		if(m_TileArray[int(heroTile.x) * m_TileCols + int(heroTile.y + 1)] == WALL) {
			float northEnd = float(int(heroTile.y + 1) * m_TileHeight * m_Start_Level.GetScale().y/*LEVEL_ONE_SCALE*/);
			float boundExtreme = boundCenter.y + boundHeight / 2;
			float overlap = northEnd - boundExtreme;
			if(overlap < 0) {
				newTranslation.y = newTranslation.y + overlap; //overlap is negative
				wall = 1;
			}
		}
	}

	//Return a translation, which may or may not have been readjusted
	return newTranslation;
}

vec3 CLevel::CheckDoorCollision(const UWB_BoundingVolume* otherBound, const vec3 otherTranslation) {
	
	UWB_Primitive* pDoor;

	vec3 newTranslation = otherTranslation;
	bool doorFlagWasSet = false;

	if(otherBound->GetType() == UWB_BoundingVolume::box) {
	
		vec3 boundCenter = otherBound->GetCenter();
		float boundWidth = static_cast<const UWB_BoundingBox*>(otherBound)->Width();
		float boundHeight = static_cast<const UWB_BoundingBox*>(otherBound)->Height();

		vec3 heroTile = boundCenter;
		heroTile.x = heroTile.x / (m_TileWidth * m_Start_Level.GetScale().x/*LEVEL_ONE_SCALE*/);
		heroTile.y = heroTile.y / (m_TileHeight * m_Start_Level.GetScale().y/*LEVEL_ONE_SCALE*/);

		//Check for a door on the tile to the left
		eTileType leftTile = m_TileArray[int(heroTile.x - 1) * m_TileCols + int(heroTile.y)];
		if(leftTile == REDDOOR) {
			pDoor = pDoorComponents->PrimitiveAt(m_RedDoorIndex);
		} else if (leftTile == GREENDOOR) {
			pDoor = pDoorComponents->PrimitiveAt(m_GreenDoorIndex);
		} else if (leftTile == BLUEDOOR) {
			pDoor = pDoorComponents->PrimitiveAt(m_BlueDoorIndex);
		} else if (leftTile == YELLOWDOOR) {
			pDoor = pDoorComponents->PrimitiveAt(m_YellowDoorIndex);
		}
		if(leftTile == REDDOOR || leftTile == GREENDOOR || leftTile == BLUEDOOR || leftTile == YELLOWDOOR) {
			if(pDoor->IsVisible()) {
				float leftEnd = float(int(heroTile.x) * m_TileWidth * m_Start_Level.GetScale().x/*LEVEL_ONE_SCALE*/);
				float boundExtreme = boundCenter.x - boundWidth / 2;
				float overlap = leftEnd - boundExtreme;
				if(overlap > 0) {
					newTranslation.x = newTranslation.x + overlap;
					m_DoorFlag = leftTile;
					doorFlagWasSet = true;
				}
			}
		}

		//Check for a door on the tile to the right
		eTileType rightTile = m_TileArray[int(heroTile.x + 1) * m_TileCols + int(heroTile.y)];
		if(rightTile == REDDOOR) {
			pDoor = pDoorComponents->PrimitiveAt(m_RedDoorIndex);
		} else if (rightTile == GREENDOOR) {
			pDoor = pDoorComponents->PrimitiveAt(m_GreenDoorIndex);
		} else if (rightTile == BLUEDOOR) {
			pDoor = pDoorComponents->PrimitiveAt(m_BlueDoorIndex);
		} else if (rightTile == YELLOWDOOR) {
			pDoor = pDoorComponents->PrimitiveAt(m_YellowDoorIndex);
		}
		if(rightTile == REDDOOR || rightTile == GREENDOOR || rightTile == BLUEDOOR || rightTile == YELLOWDOOR) {
			if(pDoor->IsVisible()) {
				float rightEnd = float(int(heroTile.x + 1) * m_TileWidth * m_Start_Level.GetScale().x/*LEVEL_ONE_SCALE*/);
				float boundExtreme = boundCenter.x + boundWidth / 2;
				float overlap = rightEnd - boundExtreme;
				if(overlap < 0) {
					newTranslation.x = newTranslation.x + overlap; //overlap is negative
					m_DoorFlag = rightTile;
					doorFlagWasSet = true;
				}
			}
		}

		//Check for a wall on the tile to the south
		eTileType southTile = m_TileArray[int(heroTile.x) * m_TileCols + int(heroTile.y - 1)];
		if(southTile == REDDOOR) {
			pDoor = pDoorComponents->PrimitiveAt(m_RedDoorIndex);
		} else if (southTile == GREENDOOR) {
			pDoor = pDoorComponents->PrimitiveAt(m_GreenDoorIndex);
		} else if (southTile == BLUEDOOR) {
			pDoor = pDoorComponents->PrimitiveAt(m_BlueDoorIndex);
		} else if (southTile == YELLOWDOOR) {
			pDoor = pDoorComponents->PrimitiveAt(m_YellowDoorIndex);
		}
		if(southTile == REDDOOR || southTile == GREENDOOR || southTile == BLUEDOOR || southTile == YELLOWDOOR) {
			if(pDoor->IsVisible()) {
				float southEnd = float(int(heroTile.y) * m_TileHeight * m_Start_Level.GetScale().y/*LEVEL_ONE_SCALE*/);
				float boundExtreme = boundCenter.y - boundHeight / 2;
				float overlap = southEnd - boundExtreme;
				if(overlap > 0) {
					newTranslation.y = newTranslation.y + overlap;
					m_DoorFlag = southTile;
					doorFlagWasSet = true;
				}
			}
		}
		
		//Check for a wall on the tile to the north
		eTileType northTile = m_TileArray[int(heroTile.x) * m_TileCols + int(heroTile.y + 1)];
		if(northTile == REDDOOR) {
			pDoor = pDoorComponents->PrimitiveAt(m_RedDoorIndex);
		} else if (northTile == GREENDOOR) {
			pDoor = pDoorComponents->PrimitiveAt(m_GreenDoorIndex);
		} else if (northTile == BLUEDOOR) {
			pDoor = pDoorComponents->PrimitiveAt(m_BlueDoorIndex);
		} else if (northTile == YELLOWDOOR) {
			pDoor = pDoorComponents->PrimitiveAt(m_YellowDoorIndex);
		}
		if(northTile == REDDOOR || northTile == GREENDOOR || northTile == BLUEDOOR || northTile == YELLOWDOOR) {
			if(pDoor->IsVisible()) {
				float northEnd = float(int(heroTile.y + 1) * m_TileHeight * m_Start_Level.GetScale().y/*LEVEL_ONE_SCALE*/);
				float boundExtreme = boundCenter.y + boundHeight / 2;
				float overlap = northEnd - boundExtreme;
				if(overlap < 0) {
					newTranslation.y = newTranslation.y + overlap; //overlap is negative
					m_DoorFlag = northTile;
					doorFlagWasSet = true;
				}
			}
		}
	}
	
	//If no door was found, set the doorFlag to EMPTY
	if(!doorFlagWasSet) {
		m_DoorFlag = EMPTY;
	}

	//Return a translation, which may or may not have been readjusted
	return newTranslation;
}

eItemType CLevel::CheckItemCollision(const UWB_BoundingVolume * otherBound) {

	for(int i = 0; i < pItemComponents->Count(); i++) {
		UWB_Primitive* pCurrentItem = pItemComponents->PrimitiveAt(i);
		const UWB_BoundingBox* pCurrentItemBounds = static_cast<const UWB_BoundingBox*>(pCurrentItem->GetBoundingVolume(theApp.GetModel().GetDrawHelper()->GetLOD() ) );

		vec3 corner1 = pCurrentItemBounds->GetMin();
		vec3 corner2 = pCurrentItemBounds->GetMax();

		UWBD3D_DrawHelper* drawHelper = theApp.GetModel().GetDrawHelper();
		drawHelper->PushModelTransform();
		GetXformInfo().SetUpModelStack(*drawHelper);
		drawHelper->TransformPoint(corner1);
		drawHelper->TransformPoint(corner2);
		drawHelper->PopModelTransform();

		const UWB_BoundingBox itemTransformed = UWB_BoundingBox(corner1, corner2);

		if(otherBound->Intersects(itemTransformed)) {
			
			//Just make the item invisible to pick it up
			if(pCurrentItem->IsVisible()) {
				pCurrentItem->SetVisible(false);

				//Find out which item was picked up
				switch(i) {
					case 0:
						return REDKEY;
					case 1:
						return BLUEKEY;
					case 2:
						return GREENKEY;
					case 3:
						return YELLOWKEY;
					case 4:
						return BATTERY;
				}
			}
		}//else the item is invisible, don't try to pick it up
	}

	//none of the items are being collided with
	return NOITEM;
}

eTileType CLevel::CheckExitCollision(const UWB_BoundingVolume* otherBound, const vec3 otherTranslation) {

	eTileType resultTile = EMPTY;

	if(otherBound->GetType() == UWB_BoundingVolume::box) {
		vec3 boundCenter = otherBound->GetCenter();
		float boundWidth = static_cast<const UWB_BoundingBox*>(otherBound)->Width();
		float boundHeight = static_cast<const UWB_BoundingBox*>(otherBound)->Height();

		vec3 heroTile = boundCenter;
		heroTile.x = heroTile.x / (m_TileWidth * m_Start_Level.GetScale().x);
		heroTile.y = heroTile.y / (m_TileHeight * m_Start_Level.GetScale().y);

		//Check for a door on the tile to the left
		eTileType leftTile = m_TileArray[int(heroTile.x - 1) * m_TileCols + int(heroTile.y)];
		if(leftTile == EXIT) {
			float leftEnd = float(int(heroTile.x) * m_TileWidth * m_Start_Level.GetScale().x);
			float boundExtreme = boundCenter.x - boundWidth / 2;
			float overlap = leftEnd - boundExtreme;
			if(overlap > 0) {
				resultTile = leftTile;
			}
		}

		//Check for a door on the tile to the right
		eTileType rightTile = m_TileArray[int(heroTile.x + 1) * m_TileCols + int(heroTile.y)];
		if(rightTile == EXIT) {
			float rightEnd = float(int(heroTile.x + 1) * m_TileWidth * m_Start_Level.GetScale().x/*LEVEL_ONE_SCALE*/);
			float boundExtreme = boundCenter.x + boundWidth / 2;
			float overlap = rightEnd - boundExtreme;
			if(overlap < 0) {
				resultTile = rightTile;
			}
		}

		//Check for a wall on the tile to the south
		eTileType southTile = m_TileArray[int(heroTile.x) * m_TileCols + int(heroTile.y - 1)];
		if(southTile == EXIT) {
			float southEnd = float(int(heroTile.y) * m_TileHeight * m_Start_Level.GetScale().y/*LEVEL_ONE_SCALE*/);
			float boundExtreme = boundCenter.y - boundHeight / 2;
			float overlap = southEnd - boundExtreme;
			if(overlap > 0) {
				resultTile = southTile;
			}
		}
		
		//Check for a wall on the tile to the north
		eTileType northTile = m_TileArray[int(heroTile.x) * m_TileCols + int(heroTile.y + 1)];
		if(northTile == EXIT) {
			float northEnd = float(int(heroTile.y + 1) * m_TileHeight * m_Start_Level.GetScale().y/*LEVEL_ONE_SCALE*/);
			float boundExtreme = boundCenter.y + boundHeight / 2;
			float overlap = northEnd - boundExtreme;
			if(overlap < 0) {
				resultTile = northTile;
			}
		}
	}
	return resultTile;
}

vec3 CLevel::CheckBridgeCollision(const UWB_BoundingVolume* otherBound, const vec3 otherTranslation) {
	
	vec3 newTranslation = otherTranslation;
	
	if(DoesLevelHaveBridges() == true && otherBound->GetType() == UWB_BoundingVolume::box) {
		vec3 boundCenter = otherBound->GetCenter();
		float boundWidth = static_cast<const UWB_BoundingBox*>(otherBound)->Width();
		float boundHeight = static_cast<const UWB_BoundingBox*>(otherBound)->Height();

		vec3 heroTile = boundCenter;
		heroTile.x = heroTile.x / (m_TileWidth * m_Start_Level.GetScale().x);
		heroTile.y = heroTile.y / (m_TileHeight * m_Start_Level.GetScale().y);

		//Check if the hero is standing on a switch. If so, then no translation is necessary, only a boolean set
		eTileType switchCheck = m_TileArray[int(heroTile.x) * m_TileCols + int(heroTile.y)];
		if(switchCheck == UNDERPASS_SWITCH || switchCheck == OVERPASS_SWITCH) {
			if(switchCheck == UNDERPASS_SWITCH) {
				m_UnderpassIsOn = true;
				return newTranslation;
			} else {
				m_UnderpassIsOn = false;
				return newTranslation;
			}
		}
		
		//No switch found, check for collision with underpass/overpass walls
		eTileType tileToCareAbout;
		if(m_UnderpassIsOn) {
			tileToCareAbout = UNDERPASS;
		} else {
			tileToCareAbout = OVERPASS;
		}

		//Check for a wall on the tile to the left
		if(m_TileArray[int(heroTile.x - 1) * m_TileCols + int(heroTile.y)] == tileToCareAbout) {
			float leftEnd = float(int(heroTile.x) * m_TileWidth * m_Start_Level.GetScale().x);
			float boundExtreme = boundCenter.x - boundWidth / 2;
			float overlap = leftEnd - boundExtreme;
			if(overlap > 0) {
				newTranslation.x = newTranslation.x + overlap;
			}
		}
		//Check for a wall on the tile to the right
		if(m_TileArray[int(heroTile.x + 1) * m_TileCols + int(heroTile.y)] == tileToCareAbout) {
			float rightEnd = float(int(heroTile.x + 1) * m_TileWidth * m_Start_Level.GetScale().x);
			float boundExtreme = boundCenter.x + boundWidth / 2;
			float overlap = rightEnd - boundExtreme;
			if(overlap < 0) {
				newTranslation.x = newTranslation.x + overlap; //overlap is negative
			}
		}
		//Check for a wall on the tile to the south
		if(m_TileArray[int(heroTile.x) * m_TileCols + int(heroTile.y - 1)] == tileToCareAbout) {
			float southEnd = float(int(heroTile.y) * m_TileHeight * m_Start_Level.GetScale().y);
			float boundExtreme = boundCenter.y - boundHeight / 2;
			float overlap = southEnd - boundExtreme;
			if(overlap > 0) {
				newTranslation.y = newTranslation.y + overlap;
			}
		}
		//Check for a wall on the tile to the north
		if(m_TileArray[int(heroTile.x) * m_TileCols + int(heroTile.y + 1)] == tileToCareAbout) {
			float northEnd = float(int(heroTile.y + 1) * m_TileHeight * m_Start_Level.GetScale().y);
			float boundExtreme = boundCenter.y + boundHeight / 2;
			float overlap = northEnd - boundExtreme;
			if(overlap < 0) {
				newTranslation.y = newTranslation.y + overlap; //overlap is negative
			}
		}
	}

	return newTranslation;
}


void CLevel::UnlockDoor(eTileType doorColor) {
	
	if(doorColor == REDDOOR) {
		UWB_Primitive* pTargetDoor = pDoorComponents->PrimitiveAt(m_RedDoorIndex);
		pTargetDoor->SetVisible(false);
	} else if (doorColor == GREENDOOR) {
		UWB_Primitive* pTargetDoor = pDoorComponents->PrimitiveAt(m_GreenDoorIndex);
		pTargetDoor->SetVisible(false);
	} else if (doorColor == BLUEDOOR) {
		UWB_Primitive* pTargetDoor = pDoorComponents->PrimitiveAt(m_BlueDoorIndex);
		pTargetDoor->SetVisible(false);
	} else if (doorColor == YELLOWDOOR) {
		UWB_Primitive* pTargetDoor = pDoorComponents->PrimitiveAt(m_YellowDoorIndex);
		pTargetDoor->SetVisible(false);
	}
}

eTileType CLevel::GetCollisionDoorColor() const {
	return m_DoorFlag;
}

bool CLevel::DoesLevelHaveBridges() const {
	return false;
}

bool CLevel::IsHeroUnderBridge() const {
	return m_UnderpassIsOn;
}

void CLevel::DrawBridgesOverhead(eLevelOfDetail lod, UWB_DrawHelper& draw_helper) const {
	//Only redraw if the level has bridges && the underpass switch is on
	if(DoesLevelHaveBridges() && m_UnderpassIsOn) {
		draw_helper.PushModelTransform();
		m_xform.SetUpModelStack( draw_helper );
		pBridgeComponents->Draw(lod, draw_helper);
	}
}

//----------PROTECTED FUNCTIONS--------//

void CLevel::MakeWall(vec3 corner1, vec3 corner2, CString texturePath) {

	vec3 adjustedMin = corner1 + vec3((float)m_TileWidth / 2, (float)m_TileHeight / 2, 0.0f);
	vec3 adjustedMax = corner2 - vec3((float)m_TileWidth / 2, (float)m_TileHeight / 2, 0.0f);

	UWB_PrimitiveRectangleXY* pWall = new UWB_PrimitiveRectangleXY();
	pWall->SetCorners(corner1, corner2);
	pWall->SetTextureFileName(texturePath);
	pWall->EnableTexturing(true);

	bool isVertical = pow(corner2.x - corner1.x, 2.0f) < pow(corner2.y - corner1.y, 2.0f) ? true : false; 

	if(isVertical) {
		int startTile = (int) adjustedMin.y / m_TileHeight;
		int endTile = (int) adjustedMax.y / m_TileHeight;
		int constantCol = (int) adjustedMin.x / m_TileWidth;
		for(int i = startTile; i <= endTile; i++) {
			m_TileArray[constantCol * m_TileCols + i] = WALL;
		}
	} else {
		int startTile = (int) adjustedMin.x / m_TileWidth;
		int endTile = (int) adjustedMax.x / m_TileWidth;
		int constantRow = (int) adjustedMin.y / m_TileHeight;
		for(int i = startTile; i <= endTile; i++) {
			m_TileArray[i * m_TileCols + constantRow] = WALL;
		}
	}
	pWallComponents->Append(pWall);
}

void CLevel::MakeDoor(vec3 corner1, vec3 corner2, /*bool isVertical,*/ eTileType color /*, UWB_PrimitiveList* doorCollection*/) {
	
	//Create and store the door primitive
	UWB_PrimitiveRectangleXY* pDoor = new UWB_PrimitiveRectangleXY();
	pDoor->SetCorners(corner1, corner2);
	pDoor->SetShadeMode( smFlat );
	
	//This is neat. The DoorIndex series can be any order thanks to this, and some doors can be left unused!
	if(color == REDDOOR) {
		pDoor->SetFlatColor(UWB_Color(1.0f, 0.0f, 0.0f, 1.0f));
		m_RedDoorIndex = pDoorComponents->Count();//doorCollection->Count();
	} else if (color == GREENDOOR) {
		pDoor->SetFlatColor(UWB_Color(0.0f, 1.0f, 0.0f, 1.0f));
		m_GreenDoorIndex = pDoorComponents->Count();//doorCollection->Count();
	} else if (color == BLUEDOOR) {
		pDoor->SetFlatColor(UWB_Color(0.0f, 0.0f, 1.0f, 1.0f));
		m_BlueDoorIndex = pDoorComponents->Count();//doorCollection->Count();
	} else if (color == YELLOWDOOR) {
		pDoor->SetFlatColor(UWB_Color(1.0f, 1.0f, 0.0f, 1.0f));
		m_YellowDoorIndex = pDoorComponents->Count();//doorCollection->Count();
	}
	pDoorComponents->Append(pDoor);//doorCollection->Append(pDoor);

	//Register door entries into the level array
	vec3 adjustedMin = corner1 + vec3((float) m_TileWidth / 2, (float)m_TileHeight / 2, 0.0f);
	vec3 adjustedMax = corner2 - vec3((float) m_TileWidth / 2, (float)m_TileHeight / 2, 0.0f);
	
	bool isVertical = pow(corner2.x - corner1.x, 2.0f) < pow(corner2.y - corner1.y, 2.0f) ? true : false; 

	if(isVertical) {
		int startTile = (int) adjustedMin.y / m_TileHeight;//4; //4.5 //25 tiles * 4 size = 100 world
		int endTile = (int) adjustedMax.y / m_TileHeight;//4; //8.5
		int constantCol = (int) adjustedMin.x / m_TileWidth;//4; //
		for(int i = startTile; i <= endTile; i++) {
			m_TileArray[constantCol * m_TileCols + i] = color;
		}
	} else {
		int startTile = (int) adjustedMin.x / m_TileWidth;//4;
		int endTile = (int) adjustedMax.x / m_TileWidth;//4;
		int constantRow = (int) adjustedMin.y / m_TileHeight;//4;
		for(int i = startTile; i <= endTile; i++) {
			m_TileArray[i * m_TileCols + constantRow] = color;
		}
	}
}

void CLevel::MakeItem(vec3 corner1, vec3 corner2, eItemType item /*, UWB_PrimitiveList* itemCollection*/) {

	UWB_PrimitiveRectangleXY* pItem = new UWB_PrimitiveRectangleXY();
	pItem->SetCorners(corner1, corner2);
	pItem->EnableBlending(true);
	pItem->EnableTexturing(true);
	
	if(item == REDKEY) {
		pItem->SetTextureFileName(L"Texture/RedKey.png"); //edited from file at http://www.dreamstime.com/keyicon-image32238
	} else if (item == GREENKEY) {
		pItem->SetTextureFileName(L"Texture/GreenKey.png"); //edited from file at http://www.dreamstime.com/keyicon-image32238
	} else if (item == BLUEKEY) {
		pItem->SetTextureFileName(L"Texture/BlueKey.png"); //edited from file at http://www.dreamstime.com/keyicon-image32238
	} else if (item == YELLOWKEY) {
		pItem->SetTextureFileName(L"Texture/YellowKey.png"); //edited from file at http://www.dreamstime.com/keyicon-image32238
	} else if (item == BATTERY) {
		pItem->SetTextureFileName(L"Texture/Battery.png"); //http://www.iconarchive.com/show/scrap-icons-by-deleket/Battery-icon.html
	}

	pItemComponents->Append(pItem);//itemCollection->Append(pItem);
}

void CLevel::MakeExit(vec3 corner1, vec3 corner2) {

	//Register an exit into the level array
	vec3 adjustedMin = corner1 + vec3((float) m_TileWidth / 2, (float)m_TileHeight / 2, 0.0f);
	vec3 adjustedMax = corner2 - vec3((float) m_TileWidth / 2, (float)m_TileHeight / 2, 0.0f);
	
	bool isVertical = pow(corner2.x - corner1.x, 2.0f) < pow(corner2.y - corner1.y, 2.0f) ? true : false; 

	if(isVertical) {
		int startTile = (int) adjustedMin.y / m_TileHeight;//4; //4.5 //25 tiles * 4 size = 100 world
		int endTile = (int) adjustedMax.y / m_TileHeight;//4; //8.5
		int constantCol = (int) adjustedMin.x / m_TileWidth;//4; //
		for(int i = startTile; i <= endTile; i++) {
			m_TileArray[constantCol * m_TileCols + i] = EXIT;
		}
	} else {
		int startTile = (int) adjustedMin.x / m_TileWidth;//4;
		int endTile = (int) adjustedMax.x / m_TileWidth;//4;
		int constantRow = (int) adjustedMin.y / m_TileHeight;//4;
		for(int i = startTile; i <= endTile; i++) {
			m_TileArray[i * m_TileCols + constantRow] = EXIT;
		}
	}
}

vec3 CLevel::GetItemCenter(eItemType guarding)
{
	vec3 itemCenter(0.0f,0.0f,0.0f);

	int index = (int)guarding -1 ; // need offset since from MakeItem REDKEY is 0
	if(index >= 0)
	{
		UWB_Primitive* pCurrentItem = pItemComponents->PrimitiveAt(index);
		if(pCurrentItem != NULL)
		{
			const UWB_BoundingBox* pCurrentItemBounds = static_cast<const UWB_BoundingBox*>(pCurrentItem->GetBoundingVolume(theApp.GetModel().GetDrawHelper()->GetLOD() ) );

			vec3 corner1 = pCurrentItemBounds->GetMin();
			vec3 corner2 = pCurrentItemBounds->GetMax();

			UWBD3D_DrawHelper* drawHelper = theApp.GetModel().GetDrawHelper();
			drawHelper->PushModelTransform();
			GetXformInfo().SetUpModelStack(*drawHelper);
			drawHelper->TransformPoint(corner1);
			drawHelper->TransformPoint(corner2);
			drawHelper->PopModelTransform();

			const UWB_BoundingBox itemTransformed = UWB_BoundingBox(corner1, corner2);
			itemCenter = itemTransformed.GetCenter();
		}
	}
	return itemCenter;
}
