//Chris Eng & Ken Rice
//CSS450 Sung

// Level.h

//Base Level class for all levels to implement

#pragma once

enum eTileType { EMPTY, WALL, REDDOOR, GREENDOOR, BLUEDOOR, YELLOWDOOR, EXIT, UNDERPASS, UNDERPASS_SWITCH, OVERPASS, OVERPASS_SWITCH};
enum eItemType { NOITEM, REDKEY, BLUEKEY, GREENKEY, YELLOWKEY, BATTERY };

class CLevel : public UWB_SceneNode {

public:

	~CLevel(); //MUST NULLIFY COMPONENT POINTERS PRIOR TO CALLING SUPERCLASS

	virtual void Initialize(); //no definition here!
	virtual int GetLevelNumber() const;

	virtual UWB_BoundingBox GetLevelDimension();
	virtual vec3 GetLevelShroudTilesDimensions();
	virtual UWB_Color GetLevelShroudColor();

	virtual bool DoesLevelHaveBridges() const;
	virtual bool IsHeroUnderBridge() const;
	virtual	void DrawBridgesOverhead(eLevelOfDetail lod, UWB_DrawHelper& draw_helper) const;

	//Get the starting transformations of each component (level is fixed)
	virtual UWB_XformInfo GetLevelStart();
	virtual UWB_XformInfo GetHeroStart();
	virtual UWB_XformInfo GetGhostStart();
	virtual UWB_XformInfo GetBlobStart();
	virtual UWB_XformInfo GetZombieStart();

	virtual vec3 CheckWallCollision(const UWB_BoundingVolume* otherBound, const vec3 otherTranslation);
	virtual vec3 CheckWallCollision(const UWB_BoundingVolume* otherBound, const vec3 otherTranslation, int& wall);
	virtual vec3 CheckDoorCollision(const UWB_BoundingVolume* otherBound, const vec3 otherTranslation);
	virtual eItemType CheckItemCollision(const UWB_BoundingVolume* otherBound);
	virtual eTileType CheckExitCollision(const UWB_BoundingVolume* otherBound, const vec3 otherTranslation);
	virtual vec3 CheckBridgeCollision(const UWB_BoundingVolume* otherBound, const vec3 otherTranslation);


	virtual eTileType GetCollisionDoorColor() const;
	virtual void UnlockDoor(eTileType doorColor);

	vec3 GetItemCenter(eItemType guarding);

protected:

	//Handled here so that subclasses don't have to, hopefully!
	virtual void MakeWall(vec3 corner1, vec3 corner2, CString texturePath);
	virtual void MakeDoor(vec3 corner1, vec3 corner2, eTileType color);
	virtual void MakeItem(vec3 corner1, vec3 corner2, eItemType item);
	virtual void MakeExit(vec3 corner1, vec3 corner2);

	//For abstracted, quick routines for making these components invisible.
	UWB_PrimitiveList* pFloorComponents; //Not actually needed, but consistent with other pointers
	UWB_PrimitiveList* pWallComponents;	 //Not actually needed, but removes a parameter from MakeWall
	UWB_PrimitiveList* pDoorComponents;
	UWB_PrimitiveList* pItemComponents;
	UWB_PrimitiveList* pBridgeComponents;

	//This is used to find the correct door primitive to turn invisible in the PrimitiveList.
	//Their order of creation does not matter, nor if they are created at all thanks to MakeDoor! 
	//However, Item components should be sorted in the order below, with Case 4: Battery.
	int m_RedDoorIndex;// = 0;
	int m_BlueDoorIndex;// = 1;
	int m_GreenDoorIndex;// = 2;
	int m_YellowDoorIndex;// = 3;
	
	//The MakeWall and MakeDoor calls will fill out the array. YOU need to initialize it according to m_TileRows/Cols!
	eTileType* m_TileArray;
	eTileType m_DoorFlag;

	//The rest of these need to be initialized by YOU!
	int m_WorldWidth; //= 100;
	int m_WorldHeight; //= 100;

	int m_TileRows;// = 25;
	int m_TileCols;// = 25;

	int m_TileHeight;// = 4;
	int m_TileWidth;// = 4;

	UWB_Color m_ShroudColor;

	bool m_UnderpassIsOn;

	UWB_XformInfo m_Start_Level;
	UWB_XformInfo m_Start_Hero;
	UWB_XformInfo m_Start_Ghost;
	UWB_XformInfo m_Start_Blob;
	UWB_XformInfo m_Start_Zombie;
};
