//Chris Eng & Ken Rice
//CSS450 Sung

//Shroud.cpp

#include "stdafx.h"
#include "CShroud.h"
#include "EscapeProject.h"

void CShroud::Initialize(int rows, int cols, float width, float height, float levelScale, float fadeRate, UWB_Color shroudColor) {

	//test
	GOOD_FRAME_RADIUS = 19.50f;
	GOOD_TILE_RANGE = 1;

	m_ShroudColor = shroudColor;

	m_Rows = rows;
	m_Cols = cols;
	m_LevelScale = levelScale;
	m_TileHeight = height / m_Rows;
	m_TileWidth = width / m_Cols;
	m_TileLife = new float[cols * rows];

	//Must pretend that a 1D array is 2D to allow dynamic allocation, or give up dynamic variables for 2D-ness
	m_DarknessTiles = new UWB_PrimitiveRectangleXY[cols * rows];
	for(int r = 0; r < m_Rows; r++) {
		for(int c = 0; c < m_Cols; c++) {

			vec3 corner1(r * m_TileWidth, c * m_TileHeight, 0.0f);
			vec3 corner2 = corner1 + vec3(m_TileWidth, m_TileHeight, 0.0f);

			UWB_PrimitiveRectangleXY currentRect = UWB_PrimitiveRectangleXY();
			currentRect.SetCorners(corner1, corner2);
			currentRect.SetShadeMode(smFlat);
			currentRect.EnableBlending(true);	//for gradual fade-in of tiles
			currentRect.EnableTexturing(false);
			currentRect.SetFlatColor(m_ShroudColor);

			m_DarknessTiles[c * m_Cols + r] = currentRect;
			m_TileLife[c * m_Cols + r] = 0.0f;
		}
	}

	//If the frame sets have already been made (possibly restarting the game...)
	if(GetChildNode(0) != NULL) {

		//Update the color of the shroud frame	
		UWB_PrimitiveList* pFrameSets = static_cast<UWB_PrimitiveList*>(GetChildNode(0)->GetPrimitive());
		
		//For every frame setting
		for(int i = 0; i < pFrameSets->Count(); i++) {
			UWB_PrimitiveList* pCurrentSet = static_cast<UWB_PrimitiveList*>(pFrameSets->PrimitiveAt(i));

			//For every primitive in the set
			for(int j = 0; j < pCurrentSet->Count(); j++) {
				UWB_Primitive* pCurrentPrim = pCurrentSet->PrimitiveAt(j);
				pCurrentPrim->SetFlatColor(m_ShroudColor);
				pCurrentPrim->SetShadingColor(UWB_Color(m_ShroudColor.PercentRed(), m_ShroudColor.PercentGreen(), m_ShroudColor.PercentBlue(), 0.0f));
			}
		}
	//Else the frames haven't been made yet, so make them now
	} else {

		UWB_PrimitiveList* pShroudTypes = new UWB_PrimitiveList();

		//List 0: cell phone -- 7 layers of primtive circles! 
		UWB_PrimitiveList* pCellPhoneFrame = new UWB_PrimitiveList();
		for(int i = 0; i < 7; i++) {
			MakeShroudFrame(vec3(0.0f, 0.0f, 0.0f), (float) GOOD_FRAME_RADIUS, NULL, m_ShroudColor, pCellPhoneFrame);
		}
		pShroudTypes->Append(pCellPhoneFrame);

		//List 1: flashlight normal -- 1 narrow wedge layered above 2 wider wedges above 1 primitive circle
		UWB_PrimitiveList* pFLNormalFrame = new UWB_PrimitiveList();
		MakeShroudFrame(vec3(0.0f, 0.0f, 0.0f), GOOD_FRAME_RADIUS, NULL, m_ShroudColor, pFLNormalFrame);		
		MakeShroudFrame(vec3(0.0f, 0.0f, 0.0f), GOOD_FRAME_RADIUS, UWB_PI / 4, m_ShroudColor, pFLNormalFrame);
		MakeShroudFrame(vec3(0.0f, 0.0f, 0.0f), GOOD_FRAME_RADIUS, UWB_PI / 6, m_ShroudColor, pFLNormalFrame);
		MakeShroudFrame(vec3(0.0f, 0.0f, 0.0f), GOOD_FRAME_RADIUS, UWB_PI / 6, m_ShroudColor, pFLNormalFrame);
		MakeShroudFrame(vec3(0.0f, 0.0f, 0.0f), GOOD_FRAME_RADIUS, UWB_PI / 8, m_ShroudColor, pFLNormalFrame);
		pShroudTypes->Append(pFLNormalFrame);

		//List 2: flashlight high -- 1 wide wedge layered above 1 wider wedge and 1 even wider wedge
		UWB_PrimitiveList* pFLHighFrame = new UWB_PrimitiveList();
		MakeShroudFrame(vec3(0.0f, 0.0f, 0.0f), GOOD_FRAME_RADIUS, UWB_PI / 2, m_ShroudColor, pFLHighFrame);
		MakeShroudFrame(vec3(0.0f, 0.0f, 0.0f), GOOD_FRAME_RADIUS, UWB_PI / 3, m_ShroudColor, pFLHighFrame);
		MakeShroudFrame(vec3(0.0f, 0.0f, 0.0f), GOOD_FRAME_RADIUS, UWB_PI / 4, m_ShroudColor, pFLHighFrame);
		pShroudTypes->Append(pFLHighFrame);

		//List four: generic mouse pan -- ???

		UWB_SceneNode* m_ShroudFrame = new UWB_SceneNode(L"Shroud Frame");
		m_ShroudFrame->SetPrimitive(pShroudTypes);
		InsertChildNode(m_ShroudFrame);

		m_TileFadeRate = 0.01f;
		SetTileFadeRate(fadeRate);
	}
}

void CShroud::RevealTile(const UWB_BoundingVolume* otherBound) {
	vec3 otherTile = otherBound->GetCenter();
	otherTile.x = otherTile.x / (m_TileWidth * m_LevelScale);
	otherTile.y = otherTile.y / (m_TileHeight * m_LevelScale);

	//Check the tiles immediately surrounding the hero (3x3)
	for(int r = (int)otherTile.x - GOOD_TILE_RANGE; r <= otherTile.x + GOOD_TILE_RANGE; r++) {
		for(int c = (int)otherTile.y - GOOD_TILE_RANGE; c <= otherTile.y + GOOD_TILE_RANGE; c++) {
			
			//Check that this [c][r] is valid
			if(-1 < r && r < m_Rows && -1 < c && c < m_Cols) {
				
				//make visible tiles within the hero's range invisible
				if(m_DarknessTiles[c * m_Cols + r].IsVisible()) {
					UWB_Color invisible(m_ShroudColor.PercentRed(), m_ShroudColor.PercentGreen(), m_ShroudColor.PercentBlue(), 0.0f);
					
					m_DarknessTiles[c * m_Cols + r].SetFlatColor(invisible);
					m_TileLife[c * m_Cols + r] = 1.0f;
				}
			}
		}
	}
}

void CShroud::SetShroudColor(UWB_Color newColor) {
	m_ShroudColor = newColor;
}
	
void CShroud::TransformShroudFrame(const vec3 newTranslation, const float newRotationRadians) {
	UWB_XformInfo frameXform = GetChildNode(0)->GetXformInfo();
	
	if(newTranslation != NULL) {
		frameXform.SetTranslation(vec3(newTranslation.x / theApp.GetModel().GetCurrentLevelScale().x, newTranslation.y / theApp.GetModel().GetCurrentLevelScale().y, 0.0f)); //the many ways to pass in newTranslation already has level one scale multiplied in, appearantly need it without
	}
	
	if(newRotationRadians != NULL) {
		frameXform.SetRotationRadians(newRotationRadians);
	}

	GetChildNode(0)->SetXformInfo(frameXform);
}

void CShroud::SetShroudIntensity(int newLevel) {
	UWB_PrimitiveList* pShroudList = static_cast<UWB_PrimitiveList*>(GetChildNode(0)->GetPrimitive());
	
	//Just turn off all shrouds, then turn on the requested one
	for(int i = 0; i < pShroudList->Count(); i++) {
		pShroudList->PrimitiveAt(i)->SetVisible(false);

		if(i == 0 && newLevel == 0) {
			pShroudList->PrimitiveAt(i)->SetVisible(true);
		} else if (i == 1 && newLevel == 1) {
			pShroudList->PrimitiveAt(i)->SetVisible(true);
		} else if (i == 2 && newLevel == 2) {
			pShroudList->PrimitiveAt(i)->SetVisible(true);
		}
	}
}
	
int CShroud::GetShroudIntensity() {
	int level = 0;

	UWB_PrimitiveList* pShroudTypes = static_cast<UWB_PrimitiveList*>(GetChildNode(0)->GetPrimitive());
	for(int i = 0; i < pShroudTypes->Count(); i++) {

		if(pShroudTypes->PrimitiveAt(i)->IsVisible()) {
			level = i;
		}
	}

	return level;
}

void CShroud::SetTileFadeRate(float newRate) {
	if(newRate >= 0.0f) {
		m_TileFadeRate = newRate;
	}
}

float CShroud::GetShroudFrameRadius() const {
	return GOOD_FRAME_RADIUS;
}

float CShroud::GetHighBeamGap() const {
	return UWB_PI / 4;
}

void CShroud::Draw( eLevelOfDetail lod, UWB_DrawHelper& draw_helper, bool drawDarkFrame, bool drawShroudTiles ) const {
	
	//If asked to draw the shroud tiles...
	if(drawShroudTiles) {
		draw_helper.PushModelTransform();
		m_xform.SetUpModelStack( draw_helper );

		//Draw the darkness grid
		for(int r = 0; r < m_Rows; r++) {
			for(int c = 0; c < m_Cols; c++) {

				if(-UWB_ZERO_TOLERANCE <= m_TileLife[c * m_Cols + r] && m_TileLife[c * m_Cols + r] <= UWB_ZERO_TOLERANCE) {
					m_TileLife[c * m_Cols + r] = 0.0f;
				} else {
					m_TileLife[c * m_Cols + r] -= m_TileFadeRate;
				}

				UWB_Color newTileColor(m_ShroudColor.PercentRed(), m_ShroudColor.PercentGreen(), m_ShroudColor.PercentBlue(), 1.0f - m_TileLife[c * m_Cols + r]);
				m_DarknessTiles[c * m_Cols + r].SetFlatColor(newTileColor);

				m_DarknessTiles[c * m_Cols + r].Draw(lod, draw_helper);
			}	
		}
		draw_helper.PopModelTransform();
	}

	//If asked to draw the shroud frame...
	if(drawDarkFrame) {
		//This call will draw the darkness following the hero
		UWB_SceneNode::Draw(lod, draw_helper);
	}
}

//Leave wedgeRadians as NULL if you want to make a Circle!
void CShroud::MakeShroudFrame(vec3 center, float radius, float wedgeRadians, UWB_Color edgeColor, UWB_PrimitiveList* gradientCollection) {
	if(wedgeRadians == NULL) {
		UWB_PrimitiveCircle* pCircle = new UWB_PrimitiveCircle();
		pCircle->SetCenter(0.0f, 0.0f, 0.0f);
		pCircle->SetFlatColor(edgeColor);
		pCircle->SetRadius(radius);
		pCircle->EnableBlending(true);
		pCircle->SetShadeMode( smGouraud );
		pCircle->SetShadingColor(UWB_Color(edgeColor.PercentRed(), edgeColor.PercentGreen(), edgeColor.PercentBlue(), 0.0f));

		gradientCollection->Append(pCircle);

	} else { 
		CWedge* pWedge = new CWedge(wedgeRadians, 0.0f);
		pWedge->SetCenter(0.0f, 0.0f, 0.0f);
		pWedge->SetFlatColor(edgeColor);
		pWedge->SetRadius(radius);
		pWedge->EnableBlending(true);
		pWedge->SetShadeMode( smGouraud );
		//pWedge->SetShadingColor(UWB_Color(0.0f, 0.0f, 0.0f, 0.0f));
		pWedge->SetShadingColor(UWB_Color(edgeColor.PercentRed(), edgeColor.PercentGreen(), edgeColor.PercentBlue(), 0.0f));

		gradientCollection->Append(pWedge);
	}
}