#include "StdAfx.h"
#include "DrawAndMouseHandler.h"
#include "EscapeProject.h"

CDrawOnlyHandler::CDrawOnlyHandler() { }

bool CDrawOnlyHandler::Initialize(CDialog& parent_dlg, int control_id)
{
	if( !m_window.ReplaceDialogControl(parent_dlg.m_hWnd, control_id) )
		return false;

	if( !m_window.AttachHandler(this) )
		return false;

	return true;
}

void CDrawOnlyHandler::DrawGraphics()
{
	if(NULL == m_pD3DDevice)
		return;

	BeginDraw();

	LoadW2NDCXform();  // implemented in D3DWindowHandler

	//clear the draw buffer to light gray
	//D3DCOLOR background = D3DCOLOR_COLORVALUE(0.8f, 0.8f, 0.8f, 0);
	D3DCOLOR background = D3DCOLOR_COLORVALUE(0.2f, 0.2f, 0.2f, 0); //I like it darker --Chris
	m_pD3DDevice->Clear(0,	// number of rectangles, 0 means the entire area
		NULL,				// rectangles to be cleared, NULL means the entire area
		D3DCLEAR_TARGET,	// clear target (color) (we are not changing the z buffer)
		background,			// new color to clear to
		0.0f,				// new Z values to clear to
		0);					// new Stencil buffer values to clear to

	//perform all of our drawing
	//theApp.GetModel().DrawModel(true, true); //draw the darkframe following the hero
	theApp.GetModel().DrawModel(true, true, CurrentlyMouseScrolling);

	EndDrawAndShow();
}

bool CDrawOnlyHandler::GetWindowClientRect(CRect& rect) const
{
	if( !IsWindow(m_window) )
		return false;

	m_window.GetClientRect(rect);
	return true;
}

/*void CDrawOnlyHandler::OnMouseMove(unsigned int nFlags, int hwX, int hwY) {
	int deviceX, deviceY;
	float wcX, wcY;
	HardwareToDevice(hwX, hwY, deviceX, deviceY);
	DeviceToWorld(deviceX, deviceY, wcX, wcY);
	
	// Echo messages
	UWB_EchoToLocationArea(CString(L"Small View"));
	
	CString hardwareMessage;
	hardwareMessage.Format(L"[%d, %d]", hwX, hwY);
	UWB_EchoToHardwareArea(hardwareMessage);
	
	CString deviceMessage;
	deviceMessage.Format(L"[%d, %d]", deviceX, deviceY);
	UWB_EchoToDeviceArea(deviceMessage);
	
	CString worldMessage;
	worldMessage.Format(L"[%0.1f, %0.1f]", wcX, wcY);
	UWB_EchoToWorldArea(worldMessage);
}*/

bool CDrawOnlyHandler::GetCurrentlyMouseScrolling() const {
	return CurrentlyMouseScrolling;
}

void CDrawOnlyHandler::SetCurrentlyMouseScrolling(bool newState) {
	CurrentlyMouseScrolling = newState;
}

//-------------------------------------------------------------

CMainHandler::CMainHandler() : m_pLinkedHandler(NULL) { }

void CMainHandler::DrawGraphics()
{
	if(NULL == m_pD3DDevice)
		return;

	BeginDraw();

	LoadW2NDCXform();

	//clear the draw buffer to light gray
	D3DCOLOR background = D3DCOLOR_COLORVALUE(0.2f, 0.2f, 0.2f, 0); //Fairly darker than 0.8f
	m_pD3DDevice->Clear(0,	// number of rectangles, 0 means the entire area
		NULL,				// rectangles to be cleared, NULL means the entire area
		D3DCLEAR_TARGET,	// clear target (color) (we are not changing the z buffer)
		background,			// new color to clear to
		0.0f,				// new Z values to clear to
		0);					// new Stencil buffer values to clear to

	//perform all of our drawing
	theApp.GetModel().DrawModel(false, false, true); //No darkframe following the hero

	//No box
	//UWBD3D_DrawHelper helper;
	//helper.InitializeModelTransform();
	//m_pLinkedHandler->DrawWCWindow(helper);

	EndDrawAndShow();
}

void CMainHandler::SetLinkedHandler( CDrawOnlyHandler* pHandler )
{
	m_pLinkedHandler = pHandler;
}

CDrawOnlyHandler* CMainHandler::GetLinkedHandler() const
{
	return m_pLinkedHandler;
}

void CMainHandler::ComputeBoundPosition(float x, float y)
{
	// Sets the m_small_view's center position to current mouse x/y position ...
	UWB_BoundingBox b = m_pLinkedHandler->GetWCWindow();
	float h = b.Height()/2.0f;
	float w = b.Width()/2.0f;
	b.SetCorners(vec3(x-w, y-h, 0.0f), vec3(x+w, y+h, 0.0f) );
	m_pLinkedHandler->SetWCWindow(b);
}

void CMainHandler::OnMouseButton(bool down, unsigned int nFlags, int hwX, int hwY)
{
	int deviceX, deviceY;
	float wcX, wcY;
	HardwareToDevice(hwX, hwY, deviceX, deviceY);
	DeviceToWorld(deviceX, deviceY, wcX, wcY);

	if(nFlags & MK_LBUTTON && down) {
		ComputeBoundPosition(wcX, wcY);
		theApp.GetModel().TransformShroudFrame(vec3(wcX, wcY, 0.0f), NULL);
		m_pLinkedHandler->SetCurrentlyMouseScrolling(true);
	}

	if(nFlags & MK_RBUTTON && down) {
		UWB_BoundingBox b = m_pLinkedHandler->GetWCWindow();
		originalWidth = b.Width();
		originalWCX = wcX;
	}
}

void CMainHandler::OnMouseMove(unsigned int nFlags, int hwX, int hwY) {
    
	int deviceX, deviceY;
	float wcX, wcY;
	HardwareToDevice(hwX, hwY, deviceX, deviceY);
	DeviceToWorld(deviceX, deviceY, wcX, wcY);

	if(nFlags & MK_LBUTTON) {
		ComputeBoundPosition(wcX, wcY);
		theApp.GetModel().TransformShroudFrame(vec3(wcX, wcY, 0.0f), NULL);
		m_pLinkedHandler->SetCurrentlyMouseScrolling(true);
	}

	if(nFlags & MK_RBUTTON) {
		//get the world coordinate window
		UWB_BoundingBox b = m_pLinkedHandler->GetWCWindow();
			
		//compute an offset to apply to the original width
		float offset = wcX - originalWCX;
		float newWidth = originalWidth + offset;

		//if the proposed box would have a legal width
		if (10.0f <= newWidth && newWidth <= 40.0f ) { //40 is the absolute max!
			//calculate variables for setting a new box
			vec3 center = b.GetCenter();
			float halfWidth = newWidth / 2;
			float halfHeight = halfWidth; //maintain a 1:1 aspect

			//set the box corners
			b.SetCorners(vec3(center.x - halfWidth, center.y - halfHeight, 0.0f), 
						 vec3(center.x + halfWidth, center.y + halfHeight, 0.0f));
				
			//update the WCWindow to the new box
			m_pLinkedHandler->SetWCWindow(b);
		}
	}
}
