// University of Washington Bothell Graphics Library
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////
#pragma once

/// Class with specific duty of drawing Direct3D graphics to an attached window
/// This implementation contains a member that represents that actual window
class CDrawOnlyHandler :  public UWBD3D_WindowHandler
{
public:
  CDrawOnlyHandler();

  /// Used to place the graphics window on a parent dialog
  /// \param parent_dlg The dialog to place the window on
  /// \param control_id ID for placeholder on dialog to replace with the graphics window
  /// \return true on success
  bool Initialize(CDialog& parent_dlg, int control_id);

  ///
  /// virtual function for drawing the actual graphics
  void DrawGraphics();

  /// Get the client rectangle of the attached window
  /// \param rect [out] on success, this is set to the client rectangle of the attached window
  /// \return true on success
  bool GetWindowClientRect(CRect& rect) const;

  //NEW ADDITION BY CHRIS //OLD FROM MP4
  //void OnMouseMove(unsigned int nFlags, int hwX, int hwY);

  bool GetCurrentlyMouseScrolling() const;
  void SetCurrentlyMouseScrolling(bool newState);

protected:
  UWBMFC_UIWindow m_window;
  
  //If scrolling, then the shroud tiles need to be drawn on the large view, too
  bool CurrentlyMouseScrolling;

};

class CMainHandler : public CDrawOnlyHandler 
{
public:
  CMainHandler();
 
  void DrawGraphics();
  void SetLinkedHandler(CDrawOnlyHandler* pHandler);
  CDrawOnlyHandler* GetLinkedHandler() const;

  void OnMouseButton(bool down, unsigned int nFlags, int windowX, int windowY);
  void OnMouseMove(unsigned int nFlags, int windowX, int windowY);

protected:
  CDrawOnlyHandler* m_pLinkedHandler;

  void ComputeBoundPosition(float x, float y);

  //for maintaining proper WCWindow resizing behavior
  float originalWidth;	//compare potential new width versus the old
  float originalWCX;	//compute a width offset using the starting RMB click's 
						//x-coordinate versus the current RMB drag's x-coordinate
};
