//Chris Eng & Ken Rice
//CSS450 Sung

// Ghost.cpp
#include "stdafx.h"
#include "Ghost.h"
#include "EscapeProject.h"

CGhost::CGhost(/*vec3 at,*/ const wchar_t* name) : UWB_SceneNode(name) {

	m_pBodyComponents  = new UWB_PrimitiveList();;

	//The body
	UWB_PrimitiveRectangleXY* pBody = new UWB_PrimitiveRectangleXY();
	pBody->SetCorners( vec3(-7,-7,0), vec3(7,7,0) );
	pBody->SetFlatColor( UWB_Color( 50,50,255));
	pBody->SetShadeMode( smFlat );
	pBody->EnableTexturing(true);
	pBody->EnableBlending(true);
	wchar_t* wstr_File = L"Texture/Ghost.png";
	pBody->SetTextureFileName(wstr_File);
	m_pBodyComponents->Append( pBody );

	//Attatch the body components as the primitive of a body SceneNode
	UWB_SceneNode* pBodyNode = new UWB_SceneNode(L"Body");
	pBodyNode->SetPrimitive(m_pBodyComponents);

	//Attatch all SceneNodes to the Ghost SceneNode
	InsertChildNode(pBodyNode);		//GetChildNode( 0 )

	m_rotation = 0.0f;
	m_timer = 0.0f;
	m_direction = GO_UP;
	m_state = gGUARDING;
	m_guarding = REDKEY;
	m_stunTimer = 0;
}

CGhost::~CGhost() {
	
	//Must nullify these pointers to avoid a crash!
	//m_pBodyComponents = NULL;

	//Now call the superclass to clean up.
	//UWB_SceneNode::~UWB_SceneNode();
}

void CGhost::SetItemOrder(eItemType key1, eItemType key2, eItemType key3, eItemType key4) {
	m_key1 = key1; 
	m_key2 = key2; 
	m_key3 = key3; 
	m_key4 = key4;

	m_guarding = m_key1;
}

void CGhost::IncrementItemGuarding()
{
	if(m_guarding == NOITEM) {
		return;
	}

	if(m_guarding == m_key1)
		m_guarding = m_key2;
	else if(m_guarding == m_key2)
		m_guarding = m_key3;
	else if(m_guarding == m_key3)
		m_guarding = m_key4;
}

eItemType CGhost::GetItemGuarding() {

	return m_guarding;
}

void CGhost::DoMove(const UWB_BoundingVolume* targetBound, vec3 itemCenter, float difficulty) {

	m_difficulty = difficulty;

	m_stunTimer--;
	if(m_stunTimer > 0)
		return;

	if(m_state == gGUARDING)
		DoGuarding(itemCenter);
	else if(m_state == gHUNTING)
		DoHunting(targetBound);
	else if(m_state == gWANDERING)
		DoWandering();
	else if(m_state == gRETURNING)
		DoReturning(itemCenter);
	else
	{
		// something bad happened to the state machine
		// return to WANDERING
		m_state = gWANDERING;
		DoWandering();
	}

	DoStateMachine(targetBound, itemCenter);
}

void CGhost::DoHunting(const UWB_BoundingVolume* targetBound) {
	
	vec3 targetCenter = targetBound->GetCenter();
	vec3 ghostCenter = GetCenterInWC();

	float dx = targetCenter.x - ghostCenter.x;
	float dy = targetCenter.y - ghostCenter.y;
	float targetAngle = atan2(dy, dx);
	WalkForwards(targetAngle);
}

void CGhost::DoGuarding(vec3 itemCenter) {

	m_xform.SetTranslation(itemCenter);
}

void CGhost::DoReturning(vec3 itemCenter) {

}

void CGhost::DoWandering() {
	
	
}

void CGhost::WalkForwards(float rotation) {
	
	float xComponent = cos(rotation) * 0.08f * m_difficulty;
	float yComponent = sin(rotation) * 0.08f * m_difficulty;

	vec3 translation = m_xform.GetTranslation();
	m_xform.SetTranslation(vec3(translation.x + xComponent, translation.y + yComponent, 0.0f));
}

void CGhost::DoStateMachine(const UWB_BoundingVolume* targetBound, vec3 itemCenter) {

	vec3 ghostCenter = GetCenterInWC();
	vec3 heroCenter = targetBound->GetCenter();
	float distance = GetDistance(ghostCenter, heroCenter);

	if(distance < 8) // do instead of "otherBound->Intersects(ghostTransformedBoundingBox)"
		theApp.GetModel().HandleHealth();

	if(distance < 20)
	{
		m_state = gHUNTING;
	}
	else if (distance > 60)
	{
		if(m_state != gGUARDING)
			IncrementItemGuarding();
		
		if(m_guarding == NOITEM)
			m_state = gHUNTING;
		else
			m_state = gGUARDING;
	}
}

float CGhost::GetDistance(vec3 start, vec3 end)
{
	float dx = (float)start.x - (float)end.x;
	float dy = (float)start.y - (float)end.y;
	float dz = (float)start.z - (float)end.z;
	float length = sqrtf(dx*dx + dy*dy + dz*dz);
	return length;
}

vec3 CGhost::GetCenterInWC() {

	UWB_PrimitiveList* pGhostComponents = static_cast<UWB_PrimitiveList*>(GetChildNode(0)->GetPrimitive());
	UWB_Primitive* pGhost = pGhostComponents->PrimitiveAt(0);
	const UWB_BoundingBox* pGhostBounds = static_cast<const UWB_BoundingBox*>(pGhost->GetBoundingVolume(theApp.GetModel().GetDrawHelper()->GetLOD() ) );
	vec3 corner1 = pGhostBounds->GetMin();
	vec3 corner2 = pGhostBounds->GetMax();
	UWBD3D_DrawHelper* drawHelper = theApp.GetModel().GetDrawHelper();
	drawHelper->PushModelTransform();
	GetXformInfo().SetUpModelStack(*drawHelper);
	drawHelper->TransformPoint(corner1);
	drawHelper->TransformPoint(corner2);
	drawHelper->PopModelTransform();
	const UWB_BoundingBox ghostTransformed = UWB_BoundingBox(corner1, corner2);
	vec3 ghostCenter = ghostTransformed.GetCenter();
	return ghostCenter;
}

int CGhost::GetStunned() const {
	return m_stunTimer;
}

void CGhost::SetStunned(int stunTimer) {
	m_stunTimer = stunTimer;
}