//Chris Eng

#include "stdafx.h"
#include "LevelThree.h"
#include "EscapeProject.h"

void CLevelThree::Initialize() {
	//Initialize all of the component arrays, they are not initialized in CLevel!
	pFloorComponents = new UWB_PrimitiveList();
	pWallComponents = new UWB_PrimitiveList();
	pDoorComponents = new UWB_PrimitiveList();
	pItemComponents = new UWB_PrimitiveList();
	pBridgeComponents = new UWB_PrimitiveList();

	//Initialize all of the size values before making anything!
	m_WorldWidth = m_WorldHeight = 108;
	m_TileRows = 27;
	m_TileCols = 27;
	m_TileHeight = m_WorldHeight / m_TileRows;
	m_TileWidth = m_WorldWidth / m_TileCols;

	//Setup the pseudo-2D array of wall tiles
	m_TileArray = new eTileType[m_TileCols * m_TileRows];
	for(int r = 0; r < m_TileRows; r++) {
		for(int c = 0; c < m_TileCols; c++) {
			m_TileArray[c * m_TileCols + r] = EMPTY;
		}
	}

	//Setup the floor
	UWB_PrimitiveRectangleXY* pFloor = new UWB_PrimitiveRectangleXY();
	pFloor->SetCorners(vec3(0.0f, 0.0f, 0.0f), vec3((float)m_WorldWidth, (float)m_WorldHeight, 0.0f));
	pFloor->SetFlatColor(UWB_Color(100.0f, 0.0f, 0.0f, 1.0f));
	pFloor->SetTextureFileName(L"Texture/GrassFloor.jpg"); //"2006-09-13-seamless-grass-20109128-slide" from http://www.d-grafix.com/textures/viewjpg.php?img=2006-09-13-seamless-grass-20109128&imgtype=seamless&category=grass&width=1024&height=1024&date=2006-09-13
	pFloor->EnableTexturing(true);
	pFloor->EnableBlending(true);
	pFloorComponents->Append(pFloor);

	UWB_PrimitiveRectangleXY* pWestArrow1 = new UWB_PrimitiveRectangleXY();
	pWestArrow1->SetCorners(vec3(16.0f, 54.0f, 0.0f), vec3(20.0f, 58.0f, 0.0f));
	pWestArrow1->SetFlatColor(UWB_Color(0.0f, 0.0f, 0.0f, 0.0f));
	pWestArrow1->SetTextureFileName(L"Texture/RightArrow.png"); //"Right Arrow Icon" from http://dryicons.com/free-icons/icons-list/stickers-icon-set/
	pWestArrow1->EnableTexturing(true);
	pWestArrow1->EnableBlending(true);
	pFloorComponents->Append(pWestArrow1);

	UWB_PrimitiveRectangleXY* pWestArrow2 = new UWB_PrimitiveRectangleXY();
	pWestArrow2->SetCorners(vec3(36.0f, 54.0f, 0.0f), vec3(40.0f, 58.0f, 0.0f));
	pWestArrow2->SetFlatColor(UWB_Color(0.0f, 0.0f, 0.0f, 0.0f));
	pWestArrow2->SetTextureFileName(L"Texture/LeftArrow.png"); //"Right Arrow Icon" from http://dryicons.com/free-icons/icons-list/stickers-icon-set/
	pWestArrow2->EnableTexturing(true);
	pWestArrow2->EnableBlending(true);
	pFloorComponents->Append(pWestArrow2);

	UWB_PrimitiveRectangleXY* pEastArrow1 = new UWB_PrimitiveRectangleXY();
	pEastArrow1->SetCorners(vec3(72.0f, 36.0f, 0.0f), vec3(76.0f, 40.0f, 0.0f));
	pEastArrow1->SetFlatColor(UWB_Color(0.0f, 0.0f, 0.0f, 0.0f));
	pEastArrow1->SetTextureFileName(L"Texture/RightArrow.png"); //"Right Arrow Icon" from http://dryicons.com/free-icons/icons-list/stickers-icon-set/
	pEastArrow1->EnableTexturing(true);
	pEastArrow1->EnableBlending(true);
	pFloorComponents->Append(pEastArrow1);

	UWB_PrimitiveRectangleXY* pEastArrow2 = new UWB_PrimitiveRectangleXY();
	pEastArrow2->SetCorners(vec3(92.0f, 36.0f, 0.0f), vec3(96.0f, 40.0f, 0.0f));
	pEastArrow2->SetFlatColor(UWB_Color(0.0f, 0.0f, 0.0f, 0.0f));
	pEastArrow2->SetTextureFileName(L"Texture/LeftArrow.png"); //"Right Arrow Icon" from http://dryicons.com/free-icons/icons-list/stickers-icon-set/
	pEastArrow2->EnableTexturing(true);
	pEastArrow2->EnableBlending(true);
	pFloorComponents->Append(pEastArrow2);




	//Draw the non-bridge stone stairways
	UWB_PrimitiveRectangleXY* pStair1 = new UWB_PrimitiveRectangleXY();
	pStair1->SetCorners(vec3(4.0f, 72.0f, 0.0f), vec3(12.0f, 104.0f, 0.0f));
	pStair1->SetTextureFileName(L"Texture/MarbleStairway.jpg"); //"Marble" from http://potatoland.com/glart/week5/images/marble.jpg
	pStair1->EnableTexturing(true);
	pFloorComponents->Append(pStair1);
	DrawStairwell(vec3(4.0f, 72.0f, 0.0f), vec3(12.0f, 104.0f, 0.0f), pFloorComponents);

	UWB_PrimitiveRectangleXY* pStair2 = new UWB_PrimitiveRectangleXY();
	pStair2->SetCorners(vec3(56.0f, 44.0f, 0.0f), vec3(80.0f, 60.0f, 0.0f));
	pStair2->SetTextureFileName(L"Texture/MarbleStairway.jpg"); //"Marble" from http://potatoland.com/glart/week5/images/marble.jpg
	pStair2->EnableTexturing(true);
	pFloorComponents->Append(pStair2);
	DrawStairwell(vec3(64.0f, 48.0f, 0.0f), vec3(80.0f, 56.0f, 0.0f), pFloorComponents);

	UWB_PrimitiveRectangleXY* pStair3 = new UWB_PrimitiveRectangleXY();
	pStair3->SetCorners(vec3(92.0f, 48.0f, 0.0f), vec3(108.0f, 56.0f, 0.0f));
	pStair3->SetTextureFileName(L"Texture/MarbleStairway.jpg"); //"Marble" from http://potatoland.com/glart/week5/images/marble.jpg
	pStair3->EnableTexturing(true);
	pFloorComponents->Append(pStair3);
	DrawStairwell(vec3(92.0f, 48.0f, 0.0f), vec3(108.0f, 56.0f, 0.0f), pFloorComponents);


	//Wall 0
	MakeWall(vec3(0.0f, 0.0f, 0.0f), vec3(104.0f, 4.0f, 0.0f), L"Texture/HedgeWall.jpg"); //"Green Grass Texture" from Shutterstock Images, http://69.90.174.248/photos/display_pic_with_logo/103244/103244,1206270548,1.jpg

	//Wall 1
	MakeWall(vec3(104.0f, 0.0f, 0.0f), vec3(108.0f, 44.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 2
	MakeWall(vec3(104.0f, 60.0f, 0.0f), vec3(108.0f, 104.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 3
	MakeWall(vec3(16.0f, 104.0f, 0.0f), vec3(108.0f, 108.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 4
	MakeWall(vec3(0.0f, 4.0f, 0.0f), vec3(4.0f, 72.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 5
	MakeWall(vec3(4.0f, 12.0f, 0.0f), vec3(12.0f, 16.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 6
	MakeWall(vec3(12.0f, 12.0f, 0.0f), vec3(16.0f, 28.0f, 0.0f), L"Texture/HedgeWall.jpg");
	
	//Wall 7
	MakeWall(vec3(16.0f, 24.0f, 0.0f), vec3(24.0f, 28.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 8
	MakeWall(vec3(4.0f, 36.0f, 0.0f), vec3(20.0f, 40.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 9
	MakeWall(vec3(32.0f, 16.0f, 0.0f), vec3(36.0f, 36.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 10
	MakeWall(vec3(24.0f, 12.0f, 0.0f), vec3(44.0f, 16.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 11
	MakeWall(vec3(32.0f, 4.0f, 0.0f), vec3(36.0f, 12.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 12
	MakeWall(vec3(44.0f, 12.0f, 0.0f), vec3(48.0f, 20.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 13
	MakeWall(vec3(36.0f, 28.0f, 0.0f), vec3(80.0f, 32.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 14
	MakeWall(vec3(56.0f, 12.0f, 0.0f), vec3(60.0f, 28.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 15
	MakeWall(vec3(60.0f, 12.0f, 0.0f), vec3(92.0f, 16.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 16
	MakeWall(vec3(68.0f, 24.0f, 0.0f), vec3(72.0f, 28.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 17
	MakeWall(vec3(80.0f, 16.0f, 0.0f), vec3(84.0f, 20.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 18
	MakeWall(vec3(92.0f, 12.0f, 0.0f), vec3(96.0f, 28.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 19
	MakeWall(vec3(88.0f, 28.0f, 0.0f), vec3(96.0f, 32.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 20
	MakeWall(vec3(92.0f, 68.0f, 0.0f), vec3(96.0f, 72.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 21
	MakeWall(vec3(92.0f, 80.0f, 0.0f), vec3(104.0f, 84.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 22
	MakeWall(vec3(92.0f, 84.0f, 0.0f), vec3(96.0f, 96.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 23
	MakeWall(vec3(80.0f, 92.0f, 0.0f), vec3(92.0f, 96.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 24
	MakeWall(vec3(68.0f, 60.0f, 0.0f), vec3(72.0f, 96.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 25
	MakeWall(vec3(80.0f, 68.0f, 0.0f), vec3(84.0f, 80.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 26
	MakeWall(vec3(72.0f, 80.0f, 0.0f), vec3(84.0f, 84.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 27
	MakeWall(vec3(56.0f, 72.0f, 0.0f), vec3(60.0f, 104.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 28
	MakeWall(vec3(44.0f, 72.0f, 0.0f), vec3(56.0f, 76.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 29
	MakeWall(vec3(44.0f, 84.0f, 0.0f), vec3(48.0f, 96.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 30
	MakeWall(vec3(32.0f, 96.0f, 0.0f), vec3(36.0f, 104.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 31
	MakeWall(vec3(24.0f, 84.0f, 0.0f), vec3(44.0f, 88.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 32
	MakeWall(vec3(32.0f, 72.0f, 0.0f), vec3(36.0f, 84.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 33
	MakeWall(vec3(16.0f, 72.0f, 0.0f), vec3(24.0f, 76.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 34
	MakeWall(vec3(0.0f, 72.0f, 0.0f), vec3(4.0f, 104.0f, 0.0f), L"Texture/StoneWall.jpg"); //"Stone Texture for Tutorial", from http://www.freeseamlesstextures.com/texture_tutorials/stone_texture_for_tutorial.jpg

	//Wall 35
	MakeWall(vec3(0.0f, 104.0f, 0.0f), vec3(16.0f, 108.0f, 0.0f), L"Texture/StoneWall.jpg");

	//Wall 36
	MakeWall(vec3(12.0f, 72.0f, 0.0f), vec3(16.0f, 104.0f, 0.0f), L"Texture/StoneWall.jpg");

	//Wall 37
	MakeWall(vec3(52.0f, 60.0f, 0.0f), vec3(68.0f, 64.0f, 0.0f), L"Texture/StoneWall.jpg");

	//Wall 38
	MakeWall(vec3(64.0f, 56.0f, 0.0f), vec3(80.0f, 60.0f, 0.0f), L"Texture/StoneWall.jpg");

	//Wall 39
	MakeWall(vec3(52.0f, 44.0f, 0.0f), vec3(56.0f, 60.0f, 0.0f), L"Texture/StoneWall.jpg");

	//Wall 40
	MakeWall(vec3(52.0f, 40.0f, 0.0f), vec3(68.0f, 44.0f, 0.0f), L"Texture/StoneWall.jpg");

	//Wall 41
	MakeWall(vec3(64.0f, 44.0f, 0.0f), vec3(80.0f, 48.0f, 0.0f), L"Texture/StoneWall.jpg");

	//Wall 42
	MakeWall(vec3(88.0f, 56.0f, 0.0f), vec3(108.0f, 60.0f, 0.0f), L"Texture/StoneWall.jpg");

	//Wall 43
	MakeWall(vec3(88.0f, 44.0f, 0.0f), vec3(108.0f, 48.0f, 0.0f), L"Texture/StoneWall.jpg");

	//The following walls will have portions of their tiles overwritten with specialize bridge tiles
	//Wall 44
	MakeWall(vec3(12.0f, 60.0f, 0.0f), vec3(44.0f, 64.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 45
	MakeWall(vec3(12.0f, 48.0f, 0.0f), vec3(44.0f, 52.0f, 0.0f), L"Texture/HedgeWall.jpg");

	//Wall 46 //JUST USING THIS TO QUICKLY SET THE ARRAY TO WALL; ACTUAL TEXTURE IN MAKEBRIDGES()
	MakeWall(vec3(20.0f, 36.0f, 0.0f), vec3(24.0f, 72.0f, 0.0f), L"Texture/StoneWall.jpg");
	pWallComponents->DeletePrimitiveAt(pWallComponents->Count()-1);

	//Wall 47 //JUST USING THIS TO QUICKLY SET THE ARRAY TO WALL; ACTUAL TEXTURE IN MAKEBRIDGES()
	MakeWall(vec3(32.0f, 36.0f, 0.0f), vec3(36.0f, 72.0f, 0.0f), L"Texture/StoneWall.jpg");
	pWallComponents->DeletePrimitiveAt(pWallComponents->Count()-1);

	//Wall 48 //JUST USING THIS TO QUICKLY SET THE ARRAY TO WALL; ACTUAL TEXTURE IN MAKEBRIDGES()
	MakeWall(vec3(76.0f, 32.0f, 0.0f), vec3(80.0f, 44.0f, 0.0f), L"Texture/StoneWall.jpg");
	pWallComponents->DeletePrimitiveAt(pWallComponents->Count()-1);

	//Wall 49 //JUST USING THIS TO QUICKLY SET THE ARRAY TO WALL; ACTUAL TEXTURE IN MAKEBRIDGES()
	MakeWall(vec3(88.0f, 32.0f, 0.0f), vec3(92.0f, 44.0f, 0.0f), L"Texture/StoneWall.jpg");
	pWallComponents->DeletePrimitiveAt(pWallComponents->Count()-1);
	
	//Super important call here!
	MakeBridges();

	//Exit tile
	MakeExit(vec3(104.0f, 48.0f, 0.0f), vec3(108.0f, 56.0f, 0.0f));

	//Red door
	MakeDoor(vec3(44.0f, 76.0f, 0.0f), vec3(48.0f, 84.0f, 0.0f), REDDOOR);

	//Green door
	MakeDoor(vec3(84.0f, 80.0f, 0.0f), vec3(92.0f, 84.0f, 0.0f), GREENDOOR);

	//Blue door
	MakeDoor(vec3(64.0f, 32.0f, 0.0f), vec3(68.0f, 40.0f, 0.0f), BLUEDOOR);

	//Yellow door
	MakeDoor(vec3(88.0f, 48.0f, 0.0f), vec3(92.0f, 56.0f, 0.0f), YELLOWDOOR);

	//Red Key
	MakeItem(vec3(98.0f, 86.0f, 0.0f), vec3(102.0f, 90.0f, 0.0f), REDKEY);

	//Green Key
	MakeItem(vec3(62.0f, 22.0f, 0.0f), vec3(66.0f, 26.0f, 0.0f), GREENKEY);

	//Blue Key
	MakeItem(vec3(6.0f, 6.0f, 0.0f), vec3(10.0f, 10.0f, 0.0f), BLUEKEY);

	//Yellow Key
	MakeItem(vec3(38.0f, 18.0f, 0.0f), vec3(42.0f, 22.0f, 0.0f), YELLOWKEY);

	//Battery
	MakeItem(vec3(58.0f, 50.0f, 0.0f), vec3(62.0f, 54.0f, 0.0f), BATTERY);

	//Add the components to this level's Primitive pointer
	UWB_PrimitiveList* pAllComponents = new UWB_PrimitiveList();
	pAllComponents->Append(pFloorComponents);
	pAllComponents->Append(pWallComponents);
	pAllComponents->Append(pDoorComponents);
	pAllComponents->Append(pItemComponents);
	pAllComponents->Append(pBridgeComponents);
	SetPrimitive(pAllComponents);

	//Define the level's starting XformInfo
	m_Start_Level.SetScale(vec3(1.75f, 1.75f, 1.0f));
	SetXformInfo(m_Start_Level);

	//Define the hero's starting XformInfo
	m_Start_Hero.SetScale(vec3(0.5f, 0.5f, 1.0f));
	m_Start_Hero.SetTranslation(vec3(8.0f * GetXformInfo().GetScale().x, 100.0f * GetXformInfo().GetScale().y, 0.0f));
	m_Start_Hero.SetRotationRadians(3 * UWB_PI / 2);

	//Define the ghost's starting XformInfo
	m_Start_Ghost.SetScale(m_Start_Hero.GetScale());
	m_Start_Ghost.SetTranslation(vec3(25.0f, 25.0f, 0.0f));
	m_Start_Ghost.SetRotationRadians(0.0f);

	//Define the blob's starting XformInfo
	m_Start_Blob.SetScale(m_Start_Hero.GetScale());
	m_Start_Blob.SetTranslation(vec3(14.0f, 30.0f, 0.0f));
	m_Start_Blob.SetRotationRadians(0.0f);

	//Define the shroud grid attributes
	m_ShroudColor = UWB_Color(0.7f, 0.7f, 0.725f, 1.0f);

	m_DoorFlag = EMPTY;
	m_UnderpassIsOn = true;
}

int CLevelThree::GetLevelNumber() const {
	return 3;
}

bool CLevelThree::DoesLevelHaveBridges() const {
	return true;
}

void CLevelThree::MakeBridges() {
	
	//First register the tiles
	//West bridge
	m_TileArray[5 * m_TileCols + 13] = OVERPASS;
	m_TileArray[5 * m_TileCols + 14] = OVERPASS;
	m_TileArray[8 * m_TileCols + 13] = OVERPASS;
	m_TileArray[8 * m_TileCols + 14] = OVERPASS;

	//East bridge
	m_TileArray[19 * m_TileCols + 8] = OVERPASS;
	m_TileArray[19 * m_TileCols + 9] = OVERPASS;
	m_TileArray[19 * m_TileCols + 10] = OVERPASS;
	m_TileArray[22 * m_TileCols + 8] = OVERPASS;
	m_TileArray[22 * m_TileCols + 9] = OVERPASS;
	m_TileArray[22 * m_TileCols + 10] = OVERPASS;

	//Overpass activation switches
	m_TileArray[4 * m_TileCols + 21] = OVERPASS_SWITCH;
	m_TileArray[5 * m_TileCols + 21] = OVERPASS_SWITCH;
	m_TileArray[20 * m_TileCols + 14] = OVERPASS_SWITCH;
	m_TileArray[21 * m_TileCols + 14] = OVERPASS_SWITCH;

	//West bridge
	m_TileArray[6 * m_TileCols + 12] = UNDERPASS;
	m_TileArray[7 * m_TileCols + 12] = UNDERPASS;
	m_TileArray[6 * m_TileCols + 15] = UNDERPASS;
	m_TileArray[7 * m_TileCols + 15] = UNDERPASS;

	//East bridge
	m_TileArray[20 * m_TileCols + 7] = UNDERPASS;
	m_TileArray[21 * m_TileCols + 7] = UNDERPASS;
	m_TileArray[20 * m_TileCols + 11] = UNDERPASS;
	m_TileArray[21 * m_TileCols + 11] = UNDERPASS;
	
	//Underpass activation switches
	m_TileArray[1 * m_TileCols + 15] = UNDERPASS_SWITCH;
	m_TileArray[2 * m_TileCols + 15] = UNDERPASS_SWITCH;
	m_TileArray[11 * m_TileCols + 15] = UNDERPASS_SWITCH;
	m_TileArray[12 * m_TileCols + 15] = UNDERPASS_SWITCH;

	//Now make the primitives associated with the above behavior
	//West bridge floor
	UWB_PrimitiveRectangleXY* pBridge1 = new UWB_PrimitiveRectangleXY();
	pBridge1->SetCorners(vec3(24.0f, 40.0f, 0.0f), vec3(32.0f, 68.0f, 0.0f));
	pBridge1->SetTextureFileName(L"Texture/MarbleStairway.jpg"); //"Marble" from http://potatoland.com/glart/week5/images/marble.jpg
	pBridge1->EnableTexturing(true);
	pBridgeComponents->Append(pBridge1);

	//Draw the steps on the floor
	DrawStairwell(vec3(24.0f, 40.0f, 0.0f), vec3(32.0f, 52.0f, 0.0f), pBridgeComponents);
	DrawStairwell(vec3(24.0f, 60.0f, 0.0f), vec3(32.0f, 72.0f, 0.0f), pBridgeComponents);

	//Redrawing of Wall 46
	UWB_PrimitiveRectangleXY* pBridge1Left = new UWB_PrimitiveRectangleXY();
	pBridge1Left->SetCorners(vec3(20.0f, 36.0f, 0.0f), vec3(24.0f, 72.0f, 0.0f));
	pBridge1Left->EnableTexturing(true);
	pBridge1Left->SetTextureFileName(L"Texture/StoneWall.jpg");
	pBridgeComponents->Append(pBridge1Left);

	//Redrawing of Wall 47
	UWB_PrimitiveRectangleXY* pBridge1Right = new UWB_PrimitiveRectangleXY();
	pBridge1Right->SetCorners(vec3(32.0f, 36.0f, 0.0f), vec3(36.0f, 72.0f, 0.0f));
	pBridge1Right->EnableTexturing(true);
	pBridge1Right->SetTextureFileName(L"Texture/StoneWall.jpg");
	pBridgeComponents->Append(pBridge1Right);

	//East bridge floor
	UWB_PrimitiveRectangleXY* pBridge2 = new UWB_PrimitiveRectangleXY();
	pBridge2->SetCorners(vec3(80.0f, 28.0f, 0.0f), vec3(88.0f, 48.0f, 0.0f));
	pBridge2->SetTextureFileName(L"Texture/MarbleStairway.jpg"); //"Marble" from http://potatoland.com/glart/week5/images/marble.jpg
	pBridge2->EnableTexturing(true);
	pBridgeComponents->Append(pBridge2);

	//Redrawing of Wall 48
	UWB_PrimitiveRectangleXY* pBridge2Left = new UWB_PrimitiveRectangleXY();
	pBridge2Left->SetCorners(vec3(76.0f, 32.0f, 0.0f), vec3(80.0f, 44.0f, 0.0f));
	pBridge2Left->EnableTexturing(true);
	pBridge2Left->SetTextureFileName(L"Texture/StoneWall.jpg");
	pBridgeComponents->Append(pBridge2Left);

	//Redrawing of Wall 49
	UWB_PrimitiveRectangleXY* pBridge2Right = new UWB_PrimitiveRectangleXY();
	pBridge2Right->SetCorners(vec3(88.0f, 32.0f, 0.0f), vec3(92.0f, 44.0f, 0.0f));
	pBridge2Right->EnableTexturing(true);
	pBridge2Right->SetTextureFileName(L"Texture/StoneWall.jpg");
	pBridgeComponents->Append(pBridge2Right);
}

void CLevelThree::DrawStairwell(vec3 corner1, vec3 corner2, UWB_PrimitiveList* pComponentCollection) {
	
	bool isVertical = pow(corner2.x - corner1.x, 2.0f) < pow(corner2.y - corner1.y, 2.0f) ? true : false; 

	if(isVertical) {
		float constLeftX = corner1.x;
		float constRightX = corner2.x;
		for(float y = corner1.y; y <= corner2.y; y += 2) {
			UWB_PrimitiveLine* pStair = new UWB_PrimitiveLine();
			pStair->SetStartPoint(constLeftX, y, 0.0f);
			pStair->SetEndPoint(constRightX, y, 0.0f);
			pStair->SetFlatColor(UWB_Color(0.5f, 0.5f, 0.5f, 1.0f));
			pStair->SetShadeMode( smFlat );
			pComponentCollection->Append(pStair);
		}
	} else {
		float constBottomY = corner1.y;
		float constTopY = corner2.y;
		for(float x = corner1.x; x <= corner2.x; x += 2) {
			UWB_PrimitiveLine* pStair = new UWB_PrimitiveLine();
			pStair->SetStartPoint(x, constBottomY, 0.0f);
			pStair->SetEndPoint(x, constTopY, 0.0f);
			pStair->SetFlatColor(UWB_Color(0.5f, 0.5f, 0.5f, 1.0f));
			pStair->SetShadeMode( smFlat );
			pComponentCollection->Append(pStair);
		}
	}

	/*	//Draw the stairs
	float constLeftX = 44.0f;
	float constRightX = 56.0f;
	for(float y = 32; y <= 52; y += 2) {
		UWB_PrimitiveLine* pStair = new UWB_PrimitiveLine();
		pStair->SetStartPoint(constLeftX, y, 0.0f);
		pStair->SetEndPoint(constRightX, y, 0.0f);
		pStair->SetFlatColor(UWB_Color(0.5f, 0.5f, 0.5f, 1.0f));
		pStair->SetShadeMode( smFlat );
		pFloorComponents->Append(pStair);
	}*/
}