//Chris Eng & Ken Rice
//CSS450 Sung

//LevelZero.cpp

#include "stdafx.h"
#include "LevelZero.h"
#include "EscapeProject.h"

void CLevelZero::Initialize() {

	m_WorldWidth = m_WorldHeight = 40;
	m_TileRows = 10;
	m_TileCols = 10;
	m_TileHeight = m_WorldHeight / m_TileRows;
	m_TileWidth = m_WorldWidth / m_TileCols;

	//Setup the pseudo-2D array of wall tiles
	m_TileArray = new eTileType[m_TileCols * m_TileRows];
	for(int r = 0; r < m_TileRows; r++) {
		for(int c = 0; c < m_TileCols; c++) {
			m_TileArray[c * m_TileCols + r] = EMPTY;
		}
	}

	//Set the floor message to the opening text, also sets the shroud color
	SetMessageType(0);

	//Draw invisible walls along the level barrier
	MakeInvisibleWall(vec3(0.0f, 0.0f, 0.0f), vec3(36.0f, 4.0f, 0.0f));
	MakeInvisibleWall(vec3(36.0f, 0.0f, 0.0f), vec3(40.0f, 36.0f, 0.0f));
	MakeInvisibleWall(vec3(4.0f, 36.0f, 0.0f), vec3(40.0f, 40.0f, 0.0f));
	MakeInvisibleWall(vec3(0.0f, 4.0f, 0.0f), vec3(4.0f, 40.0f, 0.0f));

	//Define the level's starting XformInfo
	m_Start_Level.SetScale(vec3(1.75f, 1.75f, 1.0f));
	SetXformInfo(m_Start_Level);

	//Define the hero's starting XformInfo
	m_Start_Hero.SetScale(vec3(0.5f, 0.5f, 1.0f));
	m_Start_Hero.SetTranslation(vec3(20.0f * m_Start_Level.GetScale().x, 20.0f * m_Start_Level.GetScale().y, 0.0f));
	m_Start_Hero.SetRotationRadians(UWB_PI / 2);
}

void CLevelZero::SetMessageType(int messageCode) {

	//Make sure the message code is valid
	if(messageCode < 0 || messageCode > 2) {
		return;
	}

	//Draw the words on the floor
	UWB_PrimitiveRectangleXY* pTitleText = new UWB_PrimitiveRectangleXY();
	pTitleText->SetCorners(vec3(0.0f, 0.0f, 0.0f), vec3(40.0f, 40.0f, 0.0f));
	pTitleText->SetShadeMode( smFlat );
	pTitleText->SetFlatColor(UWB_Color(0.0f, 0.0f, 0.0f, 0.0f));
	pTitleText->EnableBlending(true);
	pTitleText->EnableTexturing(true);

	switch(messageCode) {
		case 0: //opening
			pTitleText->SetTextureFileName(L"Texture/OpeningText.png");
			m_ShroudColor = UWB_Color(0.0f, 0.0f, 0.0f, 1.0f);
			break;
		case 1: //heaven
			pTitleText->SetTextureFileName(L"Texture/HeavenText.png");
			m_ShroudColor = UWB_Color(1.0f, 1.0f, 1.0f, 1.0f);		
			break;
		case 2: //ending
			pTitleText->SetTextureFileName(L"Texture/EndingText.png"); //"GiftIcon01_Cake" from http://www.camfrog.com/images/gifts/GiftIcon01_cake.png
			m_ShroudColor = UWB_Color(0.0f, 0.0f, 0.0f, 1.0f);
			break;
	};

	SetPrimitive(pTitleText);
}

void CLevelZero::MakeInvisibleWall(vec3 corner1, vec3 corner2) {

	//Just resgister wall tiles into the level array, the boundaries will be invisible
	vec3 adjustedMin = corner1 + vec3((float) m_TileWidth / 2, (float)m_TileHeight / 2, 0.0f);
	vec3 adjustedMax = corner2 - vec3((float) m_TileWidth / 2, (float)m_TileHeight / 2, 0.0f);
	
	bool isVertical = pow(corner2.x - corner1.x, 2.0f) < pow(corner2.y - corner1.y, 2.0f) ? true : false; 

	if(isVertical) {
		int startTile = (int) adjustedMin.y / m_TileHeight;
		int endTile = (int) adjustedMax.y / m_TileHeight;
		int constantCol = (int) adjustedMin.x / m_TileWidth;
		for(int i = startTile; i <= endTile; i++) {
			m_TileArray[constantCol * m_TileCols + i] = WALL;
		}
	} else {
		int startTile = (int) adjustedMin.x / m_TileWidth;
		int endTile = (int) adjustedMax.x / m_TileWidth;
		int constantRow = (int) adjustedMin.y / m_TileHeight;
		for(int i = startTile; i <= endTile; i++) {
			m_TileArray[i * m_TileCols + constantRow] = WALL;
		}
	}
}