//Chris Eng & Ken Rice
//CSS450 Sung

//Model.cpp

#include "stdafx.h"
#include "Model.h"
#include "LevelZero.h"
#include "LevelOne.h"
#include "LevelTwo.h"
#include "LevelThree.h"

CModel::CModel() : m_Hero(L"Hero"),  m_Ghost(L"Ghost") {
	m_difficulty = m_WaitingDifficulty = 1.0f;
	m_AmbientSoundCounter = 0;
	Initialize(0);
}

void CModel::Initialize(int levelNumber) {

	int oldHealth;

	//Delete any levels that may be already initialized
	if(0 <= levelNumber && levelNumber <= 3 && pCurrentLevel != NULL) {
		oldHealth = m_Health;
		pCurrentLevel = NULL; // need to NULL for Release else crash, Debug ok
		delete pCurrentLevel;
		//delete the monsters?
		m_NewLevelCreated = true;
	}

	//Allocate a new level
	if(levelNumber == 0) {
		pCurrentLevel = new CLevelZero();
	} else if(levelNumber == 1) {
		pCurrentLevel = new CLevelOne();
		//Clicking new game will flow to here; transfer waiting difficulty to active difficulty
		m_difficulty = m_WaitingDifficulty;
	} else if (levelNumber == 2) {
		pCurrentLevel = new CLevelTwo();
	} else if (levelNumber == 3) {
		pCurrentLevel = new CLevelThree();
	}

	pCurrentLevel->Initialize();
	m_Hero.SetXformInfo(pCurrentLevel->GetHeroStart());
	m_Hero.EquipFlashlight();

	if(pCurrentLevel->GetLevelNumber() != 0) {
		m_Ghost.SetXformInfo(pCurrentLevel->GetGhostStart());
		m_Blob.SetXformInfo(pCurrentLevel->GetBlobStart());
	}

	m_Shroud.Initialize((int)pCurrentLevel->GetLevelShroudTilesDimensions().x, (int)pCurrentLevel->GetLevelShroudTilesDimensions().y, pCurrentLevel->GetLevelDimension().Width(), pCurrentLevel->GetLevelDimension().Height(), pCurrentLevel->GetXformInfo().GetScale().x, SLOW_FADE *  pow(m_difficulty, 2.0f), pCurrentLevel->GetLevelShroudColor());
	m_Shroud.SetXformInfo(pCurrentLevel->GetXformInfo());
	m_Shroud.SetShroudIntensity(1);

	m_BatteryLife = MAX_BATTERY; //1000 is pretty brisk, 10000 a little slow
	
	//if this new level is a later level
	if(pCurrentLevel->GetLevelNumber() > 1) {
		//carry over the old life total, plus a little bonus
		m_Health = UWB_Round(oldHealth + 40 / m_difficulty);
		if(m_Health > 100) {
			m_Health = 100;
		}
	} else {
		//give the hero a full lifebar
		m_Health = 100; //switched to 100 by Chris; life is now taken in 20 * difficulty chunks.
	}

	m_HeroHasRedKey = m_HeroHasGreenKey = m_HeroHasBlueKey = m_HeroHasYellowKey = false;

	SetItemOrder();	
}

void CModel::SetWorldBounds(const UWB_BoundingBox newBounds) {
	m_WorldBounds = newBounds; 
}

UWB_BoundingBox CModel::GetWorldBounds() /*const*/ {
	
	UWB_BoundingBox bounds;
		
	//This pointer and static-cast is now necessary due to pCurrentLevel convention, otherwise small map view is incorrect
	const UWB_BoundingBox* levelBounds = static_cast<const UWB_BoundingBox*>(pCurrentLevel->GetPrimitive()->GetBoundingVolume(lodLow));
	vec3 corner1 = levelBounds->GetMin();
	vec3 corner2 = levelBounds->GetMax();

	m_DrawHelper.PushModelTransform();
	pCurrentLevel->GetXformInfo().SetUpModelStack(m_DrawHelper);

	m_DrawHelper.TransformPoint(corner1);
	m_DrawHelper.TransformPoint(corner2);
	m_DrawHelper.PopModelTransform();

	bounds.SetCorners(corner1, corner2);
	SetWorldBounds(bounds);

	m_NewLevelCreated = false;

	return m_WorldBounds;
}

vec3 CModel::GetCurrentLevelScale() const {
	return pCurrentLevel->GetXformInfo().GetScale();
}

void CModel::UpdateModel() {

	//Wake up the hero to perform any necessary animations
	m_Hero.Animate();

	//Allow shroud frame to do its animation when the hero is standing still
	if(m_Hero.IsEquipAnimationActive()) {
		UWB_XformInfo equipmentXform = m_Hero.GetEquipmentXform(m_DrawHelper);
		TransformShroudFrame(equipmentXform.GetTranslation(), equipmentXform.GetRotationInRadians());
	}

	//Check if the hero is colliding with a wall
	HandleHeroWallCollision();
	
	//Check if the hero is colliding with a shroud tile
	HandleHeroShroudCollision();

	//If on a special level, don't bother doing any of these
	if(pCurrentLevel->GetLevelNumber() > 0) {
		//Check if the hero is colliding with a door
		HandleHeroDoorCollision();

		//Check if the hero is colliding with an item
		HandleHeroItemCollision();

		//Check if the hero is colliding with specialize bridge tiles
		HandleHeroBridgeCollision();

		HandleBatteryLife();

		const UWB_BoundingVolume* heroBound = &m_Hero.GetBounds(m_DrawHelper, false);
		vec3 itemCenter = pCurrentLevel->GetItemCenter(m_Ghost.GetItemGuarding());
		m_Ghost.DoMove(heroBound, itemCenter, m_difficulty);
	
		m_Blob.DoMove(heroBound, itemCenter, m_difficulty);
		HandleHeroBlobCollision();
		HandleBlobWallCollision();
		HandleBlobDoorCollision();

		HandleFlashlightMonsterCollision();

		// >:)
		HandleAmbientSound();

		//PUT THIS LAST, SO THAT THE LEVEL ISN'T SWAPPED FROM UNDERNEATH THE MONSTERS!
		//Check if the hero is colliding with an exit
		HandleHeroExitCollision();
	}
}

void CModel::DrawModel(bool drawShroudFrame, bool redrawBridges, bool drawShroudTiles) {
	eLevelOfDetail lod = m_DrawHelper.GetLOD();

	m_DrawHelper.InitializeModelTransform();
	m_DrawHelper.PushModelTransform();
  
	//Draw the level
	pCurrentLevel->Draw(lod, m_DrawHelper);

	//Draw the hero if not invincible, or if on the correct cycle of the invincibility timer
	if(m_Hero.GetInvincibleTimer() % 20 <= 10) {	
		m_Hero.Draw(lod, m_DrawHelper);
	}

	//Only draw monsters if not on the opening or ending screen
	if(pCurrentLevel->GetLevelNumber() != 0) {
		//Draw the monsters only if they are the correct side of blinking
		if(m_Ghost.GetStunned() % 20 <= 10) {
			m_Ghost.Draw(lod, m_DrawHelper);
		}
		if(m_Blob.GetStunned() % 20 <= 10) {
			m_Blob.Draw(lod, m_DrawHelper);
		}
	}

	//Draw a part or both of the shroud
	m_Shroud.Draw(lod, m_DrawHelper, drawShroudFrame, drawShroudTiles);

	//Check if a bridge needs to be drawn over the hero
	if(redrawBridges && pCurrentLevel->DoesLevelHaveBridges() && pCurrentLevel->IsHeroUnderBridge()) {
		pCurrentLevel->DrawBridgesOverhead(lod, m_DrawHelper);
	}

	m_DrawHelper.PopModelTransform();
}

void CModel::HeroStepForward() {
	m_Hero.StepForward();
}

void CModel::HeroStepBackward() {
	m_Hero.StepBackward();
}
void CModel::HeroRotateCCW() {
	m_Hero.RotateCCW();
}

void CModel::HeroRotateCW() {
	m_Hero.RotateCW();
}

void CModel::HeroEquipFlashlight() {
	//This supercedes the hero's preventative measures
	if(m_Hero.IsEquipAnimationActive() == false && m_BatteryLife > 0) {
		
		PlaySound(L"Sound/FlashlightClick.wav", NULL, SND_ASYNC); //"Foley Flashlight Clicking 01" by SFX Bible, http://www.soundsnap.com 

		//flashlight not equipped, pull it out
		if(m_Hero.GetCurrentEquipment() == CELLPHONE) {
			m_Hero.EquipFlashlight();
			m_Shroud.SetShroudIntensity(1);
			m_Shroud.SetTileFadeRate(NORMAL_FADE * pow(m_difficulty, 2.0f));
		
		//else flashlight equipped, toggle modes 
		} else {
			m_Hero.ToggleFlashlightMode();

			if(m_Hero.GetCurrentEquipment() == FLASHLIGHT_NORMAL) {
				m_Shroud.SetShroudIntensity(1);
				m_Shroud.SetTileFadeRate(NORMAL_FADE *  pow(m_difficulty, 2.0f));
			} else {
				m_Shroud.SetShroudIntensity(2);
				m_Shroud.SetTileFadeRate(SLOW_FADE *  pow(m_difficulty, 2.0f));
			}	
		}
	}
}

void CModel::HeroEquipCellPhone() {
	//This supercedes the hero's preventative measures
	//Only allow the equip to happen if 1) not currently equipping, and 2) not already holding the cell phone
	if(m_Hero.IsEquipAnimationActive() == false && m_Hero.GetCurrentEquipment() != CELLPHONE) {
		PlaySound(L"Sound/CellPhone.wav", NULL, SND_ASYNC); //"Mobile Phone 4" by Justine Angus, http://www.soundsnap.com 
		m_Hero.EquipCellPhone();
		m_Shroud.SetShroudIntensity(0);
		m_Shroud.SetTileFadeRate(FAST_FADE *  pow(m_difficulty, 2.0f));
	}
}

void CModel::HeroIsWalking(bool flag) {
	m_Hero.SetIsWalking(flag);
}

UWB_XformInfo CModel::GetHeroXform() const {
	return m_Hero.GetXformInfo();
}

UWB_XformInfo CModel::GetEquipmentXform() {		
	return m_Hero.GetEquipmentXform(m_DrawHelper);
}

int CModel::GetBatteryLife() const {
	return m_BatteryLife;
}

UWBD3D_DrawHelper* CModel::GetDrawHelper() {
	
	UWBD3D_DrawHelper* helperReference = &m_DrawHelper;
	
	return helperReference;
}

void CModel::HandleHeroBlobCollision() {

	const UWB_BoundingVolume* heroBound = &m_Hero.GetBounds(m_DrawHelper, false);
	vec3 newTranslation = m_Blob.CheckHeroCollision(heroBound, m_Hero.GetXformInfo().GetTranslation());

	UWB_XformInfo heroXform = m_Hero.GetXformInfo();
	heroXform.SetTranslation(newTranslation);
	m_Hero.SetXformInfo(heroXform);
}

void CModel::HandleBlobWallCollision() {
	
	const UWB_BoundingVolume* blobBound = &m_Blob.GetBounds(m_DrawHelper, false);
	int wall = 0; 
	vec3 newTranslation = pCurrentLevel->CheckWallCollision(blobBound, m_Blob.GetXformInfo().GetTranslation(), wall);
	if(wall != 0)
		m_Blob.SetWhichWallHit(wall); 

	UWB_XformInfo blobXform = m_Blob.GetXformInfo();
	blobXform.SetTranslation(newTranslation);
	m_Blob.SetXformInfo(blobXform);
}

void CModel::HandleBlobDoorCollision() {

	const UWB_BoundingVolume* blobBound = &m_Blob.GetBounds(m_DrawHelper, false);
	vec3 newTranslation = pCurrentLevel->CheckDoorCollision(blobBound, m_Blob.GetXformInfo().GetTranslation());

	UWB_XformInfo blobXform = m_Blob.GetXformInfo();
	blobXform.SetTranslation(newTranslation);
	m_Blob.SetXformInfo(blobXform);
}

//helper for UpdateModel
void CModel::HandleHeroWallCollision() {
	
	const UWB_BoundingVolume* heroBound = &m_Hero.GetBounds(m_DrawHelper, false);
	vec3 newTranslation = pCurrentLevel->CheckWallCollision(heroBound, m_Hero.GetXformInfo().GetTranslation());

	UWB_XformInfo heroXform = m_Hero.GetXformInfo();
	heroXform.SetTranslation(newTranslation);
	m_Hero.SetXformInfo(heroXform);
}

void CModel::HandleHeroDoorCollision() {

	const UWB_BoundingVolume* heroBound = &m_Hero.GetBounds(m_DrawHelper, false);
	vec3 newTranslation = pCurrentLevel->CheckDoorCollision(heroBound, m_Hero.GetXformInfo().GetTranslation());

	UWB_XformInfo heroXform = m_Hero.GetXformInfo();
	heroXform.SetTranslation(newTranslation);
	m_Hero.SetXformInfo(heroXform);

	eTileType doorColor = pCurrentLevel->GetCollisionDoorColor();
	if (doorColor != EMPTY) {
		if(doorColor == REDDOOR && m_HeroHasRedKey) {
			m_HeroHasRedKey = false;
			pCurrentLevel->UnlockDoor(doorColor);
			PlaySound(L"Sound/DoorUnlock.wav", NULL, SND_ASYNC); //door unlock by Justine Angus, http://www.soundsnap.com
		} else if(doorColor == GREENDOOR && m_HeroHasGreenKey) {
			m_HeroHasGreenKey = false;
			pCurrentLevel->UnlockDoor(doorColor);
			PlaySound(L"Sound/DoorUnlock.wav", NULL, SND_ASYNC); //door unlock by Justine Angus, http://www.soundsnap.com
		} else if(doorColor == BLUEDOOR && m_HeroHasBlueKey) {
			m_HeroHasBlueKey = false;
			pCurrentLevel->UnlockDoor(doorColor);
			PlaySound(L"Sound/DoorUnlock.wav", NULL, SND_ASYNC); //door unlock by Justine Angus, http://www.soundsnap.com
		} else if(doorColor == YELLOWDOOR && m_HeroHasYellowKey) {
			m_HeroHasYellowKey = false;
			pCurrentLevel->UnlockDoor(doorColor);
			PlaySound(L"Sound/DoorUnlock.wav", NULL, SND_ASYNC); //door unlock by Justine Angus, http://www.soundsnap.com
		} else {
			//Would try to play a "door locked" sound here, but the hero could trigger it repeatedly and cause slowdown :(
		}
	}
}

void CModel::HandleHeroItemCollision() {
	const UWB_BoundingVolume* heroBound = &m_Hero.GetBounds(m_DrawHelper, false);
	eItemType item = pCurrentLevel->CheckItemCollision(heroBound);

	if(item != NOITEM) {
		PlaySound(L"Sound/CollectItem.wav", NULL, SND_ASYNC); //"Nunchaku pick up" by Blastwave FX on http://www.soundsnap.com
		
		if(item == REDKEY) {
			m_HeroHasRedKey = true;
		} else if (item == GREENKEY) {
			m_HeroHasGreenKey = true;
		} else if (item == BLUEKEY) {
			m_HeroHasBlueKey = true;
		} else if (item == YELLOWKEY) {
			m_HeroHasYellowKey = true;
		} else if (item == BATTERY) {
			m_BatteryLife = MAX_BATTERY;
		}
	}
}

void CModel::HandleHeroShroudCollision() {

	//THIS IS CALLED WHEN THE DIALOG GETS A DIRECTIONAL KEY OR LEFT CLICK, NOT HERE ANYMORE
	//also handles the darkness frame following the hero
	//m_Darkness.TranslateDarkness(GetHeroPosition());

	//handle the shroud tiles
	const UWB_BoundingVolume* heroBound = &m_Hero.GetBounds(m_DrawHelper, false);
	m_Shroud.RevealTile(heroBound);
}

void CModel::HandleHeroExitCollision() {
	const UWB_BoundingVolume* heroBound = &m_Hero.GetBounds(m_DrawHelper, false);
	eTileType resultTile = pCurrentLevel->CheckExitCollision(heroBound, m_Hero.GetXformInfo().GetTranslation());
	
	if(resultTile == EXIT) {
		int nextLevel = (pCurrentLevel->GetLevelNumber() + 1) % 4;

		if(nextLevel != 0) {
			Initialize(nextLevel);
		} else {
			Initialize(nextLevel);
			CLevelZero* pTemp = static_cast<CLevelZero*>(pCurrentLevel);
			pTemp->SetMessageType(2);
			//Manually reset the shroud, given this special situation
			m_Shroud.Initialize((int)pCurrentLevel->GetLevelShroudTilesDimensions().x, (int)pCurrentLevel->GetLevelShroudTilesDimensions().y, pCurrentLevel->GetLevelDimension().Width(), pCurrentLevel->GetLevelDimension().Height(), pCurrentLevel->GetXformInfo().GetScale().x, SLOW_FADE, pCurrentLevel->GetLevelShroudColor());
			//PlaySound(L"Congrats.wav", NULL, SND_ASYNC);
		}
	}
}

void CModel::HandleHeroBridgeCollision() {
	//Only proceed if the current level has bridges
	if(pCurrentLevel->DoesLevelHaveBridges()) {
		const UWB_BoundingVolume* heroBound = &m_Hero.GetBounds(m_DrawHelper, false);
		vec3 newTranslation = pCurrentLevel->CheckBridgeCollision(heroBound, m_Hero.GetXformInfo().GetTranslation());

		UWB_XformInfo heroXform = m_Hero.GetXformInfo();
		heroXform.SetTranslation(newTranslation);
		m_Hero.SetXformInfo(heroXform);
	}
}

bool CModel::GetPauseState() const {
	return m_IsPaused;
}
void CModel::SetPauseState(bool flag) {
	m_IsPaused = flag;
	PlaySound(L"Sound/PauseBlip.wav", NULL, SND_ASYNC); //"Alien Control Blip 01" by Sound Bible at http://www.soundsnap.com 
}

//Initialize sets this to true. GetWorldBounds sets this to false. Dialog can call GetIsNewLevelCreated to see if it needs to resize the
//WCWindow of the small view via GetWorldBounds after.
bool CModel::GetIsNewLevelCreated() {
	return m_NewLevelCreated;
}

void CModel::HandleBatteryLife() {
	eEquipmentType currentTool = m_Hero.GetCurrentEquipment();

	if(currentTool == FLASHLIGHT_NORMAL) {
		m_BatteryLife -= UWB_Round(1 * m_difficulty);
	} else if (currentTool == FLASHLIGHT_HIGH) {
		m_BatteryLife -= UWB_Round(3 * m_difficulty);
	}

	if(m_BatteryLife <= 0) {
		m_BatteryLife = 0;
		HeroEquipCellPhone();
	}
}

bool CModel::HeroHasRedKey() const
{
	return m_HeroHasRedKey;
}

bool CModel::HeroHasBlueKey() const
{
	return m_HeroHasBlueKey;
}

bool CModel::HeroHasGreenKey() const
{
	return m_HeroHasGreenKey;
}

bool CModel::HeroHasYellowKey() const
{
	return m_HeroHasYellowKey;
}

void CModel::TransformShroudFrame(vec3 newTranslation, float newRotation) {
	m_Shroud.TransformShroudFrame(newTranslation, newRotation);
}

int CModel::GetHealth() const {
	return m_Health;
}

void CModel::HandleHealth() {
	
	//Only decrement health if the hero isn't invincible
	if(m_Hero.GetInvincibleTimer() == 0) {
		
		//Decrement some health from the hero with influence from the difficulty
		m_Health = UWB_Round(m_Health - 20 * m_difficulty);

		PlaySound(L"Sound/HeroHit.wav", NULL, SND_ASYNC); //"Male3 Hurt1" by Rebecca Parnell at http://www.soundsnap.com

		//If the hero's life has expired...
		if(m_Health <= 0) {
			//Go to the afterlife...
			Initialize(0);
			CLevelZero* pTemp = static_cast<CLevelZero*>(pCurrentLevel);
			pTemp->SetMessageType(1); //HeavenText
			m_Shroud.Initialize((int)pCurrentLevel->GetLevelShroudTilesDimensions().x, (int)pCurrentLevel->GetLevelShroudTilesDimensions().y, pCurrentLevel->GetLevelDimension().Width(), pCurrentLevel->GetLevelDimension().Height(), pCurrentLevel->GetXformInfo().GetScale().x, SLOW_FADE, pCurrentLevel->GetLevelShroudColor());
			PlaySound(L"Sound/EvilLaugh.wav", NULL, SND_ASYNC); //"Evil Laughing.wav" by timtube at http://www.freesound.org/samplesViewSingle.php?id=61042
		
		//Else the hero is still alive
		} else {
			//Set the hero to invincible for a period influenced by the difficulty
			m_Hero.SetInvincibleTimer(UWB_Round(120 / m_difficulty));	
		}
	}
}

float CModel::GetWaitingDifficulty() const {
	return m_WaitingDifficulty;
}

void CModel::SetWaitingDifficulty(float newDifficulty) {
	if(newDifficulty > 0.0f) {
		m_WaitingDifficulty = newDifficulty;
	}
}

CLevel* CModel::GetCurrentLevel() const {

	return pCurrentLevel;
}

void CModel::SetItemOrder() {

	if(pCurrentLevel->GetLevelNumber() == 1)
		m_Ghost.SetItemOrder(REDKEY, BLUEKEY, GREENKEY, NOITEM);
	if(pCurrentLevel->GetLevelNumber() == 2)
		m_Ghost.SetItemOrder(REDKEY, GREENKEY, BLUEKEY, YELLOWKEY);
	if(pCurrentLevel->GetLevelNumber() == 1)
		m_Ghost.SetItemOrder(REDKEY, BLUEKEY, GREENKEY, YELLOWKEY);

}

void CModel::HandleFlashlightMonsterCollision() {
	
	//placeholder code for now
	if(m_Hero.GetCurrentEquipment() == FLASHLIGHT_HIGH) {

		vec3 ghostCenter = m_Ghost.GetBounds(m_DrawHelper, false).GetCenter();
		vec3 blobCenter = m_Blob.GetBounds(m_DrawHelper, false).GetCenter();
		vec3 heroCenter = m_Hero.GetBounds(m_DrawHelper, false).GetCenter();
		float onscreenRadius = m_Shroud.GetShroudFrameRadius();
		
		float heroGhostDist = sqrt(pow(heroCenter.x - ghostCenter.x, 2.0f) + pow(heroCenter.y - ghostCenter.y, 2.0f));
		float heroBlobDist = sqrt(pow(heroCenter.x - blobCenter.x, 2.0f) + pow(heroCenter.y - blobCenter.y, 2.0f));
		float stunTimer = 160.0f/m_difficulty;

		//only stun the monsters if they are onscreen
		if(heroGhostDist < onscreenRadius && m_Ghost.GetStunned() <= 0) {
			m_Ghost.SetStunned((int)stunTimer);
			PlaySound(L"Sound/MonsterHit", NULL, SND_ASYNC); //"Growling Bear Monster" by BlastwaveFX at http://freesound.com 
		}
		if(heroBlobDist < onscreenRadius && m_Blob.GetStunned() <= 0) {
			m_Blob.SetStunned((int)stunTimer);
			PlaySound(L"Sound/MonsterHit", NULL, SND_ASYNC); //"Growling Bear Monster" by BlastwaveFX at http://freesound.com 
		}
	}

	//Only check to stun a monster if the Hero's flashlight is on high
	/*if(m_Hero.GetCurrentEquipment() == FLASHLIGHT_HIGH) {

		//Get the location & rotation of the beam
		UWB_XformInfo beamXform = m_Hero.GetEquipmentXform(m_DrawHelper);
		//Get beam length
		float remainingRadius = m_Shroud.GetShroudFrameRadius();
		//Get theta (radians)
		float theta = m_Shroud.GetHighBeamGap();

		//strategy: a series of circles(with bounding boxes...) of radius approximately the
		//1/2 the size of the arc length of where it is in the beam.
		
		float boundRadius1 = UWB_VERY_LARGE;
		float positionOnBeam1 = remainingRadius;
		while(positionOnBeam1 + boundRadius1 > remainingRadius) {
			positionOnBeam1 -= 0.5f;
			boundRadius1 = theta * positionOnBeam1 / 2;
		}
		remainingRadius -= boundRadius1 * 2;

		float boundRadius2 = UWB_VERY_LARGE;
		float positionOnBeam2 = remainingRadius;
		while(positionOnBeam2 + boundRadius2 > remainingRadius) {
			positionOnBeam2 -= 0.5f;
			boundRadius2 = theta * positionOnBeam2 / 2;
		}
		remainingRadius -= boundRadius2 * 2;

		float boundRadius3 = UWB_VERY_LARGE;
		float positionOnBeam3 = remainingRadius;
		while(positionOnBeam3 + boundRadius3 > remainingRadius) {
			positionOnBeam3 -= 0.5f;
			boundRadius3 = theta * positionOnBeam3 / 2;
		}

		UWB_PrimitiveCircle largeCirc = UWB_PrimitiveCircle();
		largeCirc.SetCenter(beamXform.GetTranslation().x + cos(theta) * positionOnBeam1, beamXform.GetTranslation().y + sin(theta) * positionOnBeam1, 0.0f);
		largeCirc.SetRadius(boundRadius1);

		UWB_PrimitiveCircle medCirc = UWB_PrimitiveCircle();
		medCirc.SetCenter(beamXform.GetTranslation().x + cos(theta) * positionOnBeam2, beamXform.GetTranslation().y + sin(theta) * positionOnBeam2, 0.0f);
		medCirc.SetRadius(boundRadius2);

		UWB_PrimitiveCircle smallCirc = UWB_PrimitiveCircle();
		smallCirc.SetCenter(beamXform.GetTranslation().x + cos(theta) * positionOnBeam3, beamXform.GetTranslation().y + sin(theta) * positionOnBeam3, 0.0f);
		smallCirc.SetRadius(boundRadius3);

		const UWB_BoundingVolume* largeBound = largeCirc.GetBoundingVolume(lodLow);
		const UWB_BoundingVolume* medBound = medCirc.GetBoundingVolume(lodLow);
		const UWB_BoundingVolume* smallBound = smallCirc.GetBoundingVolume(lodLow);

		UWB_BoundingBox blobBound = m_Blob.GetBounds(m_DrawHelper, false);
		UWB_BoundingBox ghostBound = m_Ghost.GetBounds(m_DrawHelper, false);

		if(largeBound->Intersects(blobBound) || medBound->Intersects(blobBound) || smallBound->Intersects(blobBound)) {
			m_Blob.SetStunned(true);	
			//m_Blob.setStunned(160 / m_Difficulty);
		}

		if(largeBound->Intersects(ghostBound) || medBound->Intersects(ghostBound) || smallBound->Intersects(ghostBound)) {
			m_Ghost.SetStunned(true);	
			//m_Ghost.SetStunned(160 / m_Difficulty);
		}
	}*/
}

void CModel::HandleAmbientSound() {
	m_AmbientSoundCounter = (m_AmbientSoundCounter + 1) % 600; //about 15 seconds?

	//599 is just some number
	if(m_AmbientSoundCounter == 599) {
		float random = UWB_RandomNumber(6.0f);
		if(UWB_Round(random) % 3 == 0) {
			PlaySound(L"Sound/AmbientHorror1.wav", NULL, SND_ASYNC); //"Sound Design Horror Accent..." by SFX Bible from http://www.freesound.com
		} else if(UWB_Round(random) % 3 == 1) {
			PlaySound(L"Sound/AmbientHorror2.wav", NULL, SND_ASYNC); //"Sound Design Horror Accent..." by Blastwave FX from http://www.freesound.com
		} else {
			PlaySound(L"Sound/AmbientHorror3.wav", NULL, SND_ASYNC); //"Spooky3" by bubaproducer from http://www.freesound.com
		}
	}
}