//Chris Eng & Ken Rice
//CSS450 Sung

//Model.h

#pragma once

//this works! Sound Stuff
#pragma comment (lib, "winmm.lib")
#include <windows.h>
#include <mmsystem.h>

#include "Human.h"

#include "CLevel.h"
#include "CShroud.h"

#include "Ghost.h"
#include "Blob.h"

class CModel {

public:
	CModel();

	void SetWorldBounds(const UWB_BoundingBox);
	UWB_BoundingBox GetWorldBounds() /*const*/;
	vec3 GetCurrentLevelScale() const; //for Darkness frame transformation...

	void UpdateModel();
	void DrawModel(bool drawShroudFrame, bool redrawBridges, bool drawShroudTiles);

	void HeroStepForward();
	void HeroStepBackward();
	void HeroRotateCCW();
	void HeroRotateCW();
	void HeroEquipFlashlight();
	void HeroEquipCellPhone();
	void HeroIsWalking(bool flag);

	UWB_XformInfo GetHeroXform() const;
	UWB_XformInfo GetEquipmentXform();

	int GetBatteryLife() const;
	
	int GetHealth() const;
	void HandleHealth();

	UWBD3D_DrawHelper* GetDrawHelper();
	
	bool GetPauseState() const;
	void SetPauseState(bool flag);

	bool GetIsNewLevelCreated();

	bool HeroHasRedKey() const;
	bool HeroHasBlueKey() const;
	bool HeroHasGreenKey() const;
	bool HeroHasYellowKey() const;

	void Initialize(int levelNumber);

	void TransformShroudFrame(vec3 newTranslation, float newRotationRadians);

	float GetWaitingDifficulty() const;
	void SetWaitingDifficulty(float newDifficulty);

	CLevel* GetCurrentLevel() const;

	void SetItemOrder();

protected:
	UWB_BoundingBox m_WorldBounds;
	UWB_Clock m_Stopwatch;
	UWBD3D_DrawHelper m_DrawHelper;

	CHuman m_Hero;
	CShroud m_Shroud;
	CGhost m_Ghost;
	CBlob m_Blob;

	bool m_IsPaused;
	int m_BatteryLife;
	static const int MAX_BATTERY = 2500;
	int m_Health;

	//test helper for dialog correctly setting the small window
	bool m_NewLevelCreated;

	//Points to a level object, which ever level it may be
	CLevel* pCurrentLevel;

	bool m_HeroHasRedKey;
	bool m_HeroHasGreenKey;
	bool m_HeroHasBlueKey;
	bool m_HeroHasYellowKey;

	float m_difficulty;
	float m_WaitingDifficulty;
	int m_AmbientSoundCounter;

	//UpdateModel() helpers
	void HandleHeroWallCollision();
	void HandleHeroDoorCollision();
	void HandleHeroItemCollision();
	void HandleHeroShroudCollision();
	void HandleHeroExitCollision();
	void HandleHeroBridgeCollision();
	void HandleBatteryLife();
	void HandleHeroBlobCollision();
	void HandleBlobWallCollision();
	void HandleBlobDoorCollision();
	void HandleFlashlightMonsterCollision();
	void HandleAmbientSound();
};
