#include "uwbgl_D3DGraphicsSystem3.h"

// items in the texture table are sorted by texture name. This is a binary
// search function that finds a resource in the list very quickly. Typically these samples
// do not contain a lot of textures, so the speed improvements of a binary search over linear
// search is minimal, but it is always best to perform a binary search if the data is sorted
static UWBD3D_TextureResource* BinarySearchTextures( const wchar_t* texture_name, UWB_PointerArray<UWBD3D_TextureResource*>& table, int span_min, int span_max )
{
  if( span_max < span_min )
    return NULL;

  int index = (span_min + span_max) / 2;
  if( index < 0 || index >= table.Count() )
    return NULL;

  UWBD3D_TextureResource* pResource = table[index];
  int compare_result = ::wcscmp( texture_name, pResource->GetResourceName() );

  // if the strings match, we found the resource
  if( 0 == compare_result )
    return pResource;

  // now we need to figure out if there are more potential candidates or if we should just bail
  if( span_max == span_min )
    return NULL;

  if( compare_result < 0 )
  {
    if( index > span_min )
      return BinarySearchTextures( texture_name, table, span_min, index-1);
  }
  else
  {
    if( index < span_max )
      return BinarySearchTextures( texture_name, table, index+1, span_max);
  }
  return NULL;
}

UWBD3D_TextureResource* UWBD3D_GraphicsSystem::FindTextureResource( const wchar_t* texture_name, bool add_on_failure )
{
  if( wcslen(texture_name) == 0 )
    return NULL;

  UWBD3D_TextureResource* pResource = BinarySearchTextures( texture_name, m_texture_resources, 0, m_texture_resources.Count()-1 );
  if( pResource )
    return pResource;

  // if we get here, the resource is not in the table attempt to add it
  if( add_on_failure )
  {
    UWBD3D_TextureResource *pEntry = new UWBD3D_TextureResource();
    if( !pEntry->LoadResource( m_pD3DDevice, texture_name ) )
    {
      delete pEntry;
      return NULL;
    }

    // figure out where to add the resource (sorted by name)
    for( int i=0; i<m_texture_resources.Count(); i++ )
    {
      UWBD3D_TextureResource* pResource = m_texture_resources[i];
      int compare_result = ::wcscmp( texture_name, pResource->GetResourceName() );
      if( compare_result < 0 )
      {
        m_texture_resources.Insert( pEntry, i );
        return pEntry;
      }
    }

    m_texture_resources.Append( pEntry );
    return pEntry;
  }

  return NULL;
}