// University of Washington Bothell Graphics Library
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////
#pragma once
#include "uwbgl_Primitive3.h"

///
/// A primitive drawing object that represents a circle
class UWB_PrimitiveCircle : public UWB_Primitive
{
public:
  /// Initialize the center at {0,0,0} and a radius of
  /// The default constructor creates a zero size circle
  UWB_PrimitiveCircle();

  /// Update the position of the circle based in it's velocity
  /// \param elapsed_seconds amount of time that has passed since the last update
  void Update(float elapsed_seconds);

  /// User mouse input handler for this primitive
  /// position is set to the given mouse input location
  void MouseDownVertex(int vertexID, float x, float y);

  /// Mouse user input handler for this primitive
  /// Move the position of the point to a new location
  void MoveTo(float x, float y);

  /// Get the current location of this circle in 3D coordinates
  /// This is considered the center of the circle
  vec3 GetLocation() const;

  /// Virtual override for getting the bounding volume of this circle
  /// This always just returns the address of m_bounds
  virtual const UWB_BoundingVolume* GetBoundingVolume( eLevelOfDetail lod ) const;

  void SetCenter(float x, float y, float z = 0.0f);
  void SetCenter(vec3 center);
  vec3 GetCenter() const;

  void SetRadius(float radius);
  float GetRadius() const;

protected:
  /// Virtual drawing routine for this primitive. All of the drawing operations
  /// are called on the draw_helper in order to abstract the graphics API from
  /// the primitive class
  void DrawPrimitive( eLevelOfDetail lod, UWB_DrawHelper& draw_helper );

  /// Virtual function used to define the current attributes for drawing
  /// \param draw_helper this object is first passed to this function for setting
  ///        up it's attribute state and then the DrawPrimitive function is called
  ///        with this same object
  void SetupDrawAttributes(UWB_DrawHelper& draw_helper);

protected:
  ///
  /// let's keep all points in 3D space with z=0
  vec3 m_center;
  ///
  /// Radius of the circle
  float m_radius;
  UWB_BoundingBox m_bounds;
};
