#include "uwbgl_PrimitiveRectXY2.h"

UWB_PrimitiveRectangleXY::UWB_PrimitiveRectangleXY()
{
  m_bounds.SetCorners( vec3(0,0,0), vec3(0,0,0) );
}

void UWB_PrimitiveRectangleXY::SetupDrawAttributes(UWB_DrawHelper& draw_helper)
{
   __super::SetupDrawAttributes(draw_helper);
  draw_helper.SetColor2(m_ShadingColor);
  draw_helper.SetShadeMode(m_ShadeMode);
  draw_helper.SetFillMode(m_FillMode);
}

void UWB_PrimitiveRectangleXY::DrawPrimitive( eLevelOfDetail lod, UWB_DrawHelper& draw_helper )
{
  draw_helper.DrawRectangleXY( m_bounds.GetMin(), m_bounds.GetMax() );
}

void UWB_PrimitiveRectangleXY::MouseDownVertex(int vertexID, float x, float y)
{
  if(vertexID == 0)
  {
    m_mouse_down_point = vec3(x,y,0);
    m_bounds.SetCorners( m_mouse_down_point, m_mouse_down_point );
  }
  else
  {
    m_bounds.SetCorners( m_mouse_down_point, vec3(x,y,0) );
  }
}

void UWB_PrimitiveRectangleXY::MoveTo(float x, float y)
{
  vec3 old_center = m_bounds.GetCenter();
  float dx = x-old_center.x;
  float dy = y-old_center.y;

  vec3 min_point = m_bounds.GetMin();
  vec3 max_point = m_bounds.GetMax();

  min_point.x += dx;
  min_point.y += dy;
  max_point.x += dx;
  max_point.y += dy;
  m_bounds.SetCorners( min_point, max_point );
}

vec3 UWB_PrimitiveRectangleXY::GetLocation() const
{
  return m_bounds.GetCenter();
}

void UWB_PrimitiveRectangleXY::SetCorners(vec3 corner1, vec3 corner2)
{
  m_bounds.SetCorners( corner1, corner2 );
}

void UWB_PrimitiveRectangleXY::Update(float elapsed_seconds)
{
  vec3 adjust_vec = m_velocity * elapsed_seconds;
  vec3 min_point = m_bounds.GetMin();
  vec3 max_point = m_bounds.GetMax();

  min_point += adjust_vec;
  max_point += adjust_vec;
  m_bounds.SetCorners( min_point, max_point );
}

const UWB_BoundingVolume* UWB_PrimitiveRectangleXY::GetBoundingVolume(eLevelOfDetail lod) const
{
  return &m_bounds;
}
