#pragma once
#include "uwbgl_Primitive3.h"

///
/// A primitive drawing object that represents a planar 2-D rectangle
class UWB_PrimitiveRectangleXY : public UWB_Primitive
{
public:
  /// Initialize the rectangle as a zero size rectangle.
  /// i.e. both corners are at 0,0,0
  UWB_PrimitiveRectangleXY();

  /// Update the position of the rectangle based in it's velocity
  /// \param elapsed_seconds amount of time that has passed since the last update
  virtual void Update(float elapsed_seconds);

  /// User mouse input handler for this primitive
  /// position is set to the given mouse input location
  virtual void MouseDownVertex(int vertexID, float x, float y);

  /// Mouse user input handler for this primitive
  /// Move the position of the point to a new location
  virtual void MoveTo(float x, float y);

  /// Get the current location of this rectangle in 3D coordinates
  /// This is considered the center of the rectangle
  vec3 GetLocation() const;

  /// Virtual override for getting the bounding volume of this circle
  /// This always just returns the address of m_bounds
  virtual const UWB_BoundingVolume* GetBoundingVolume( eLevelOfDetail lod ) const;

  void SetCorners(vec3 corner1, vec3 corner2);
  vec3 MinCorner() const;
  vec3 MaxCorner() const;

protected:
  /// Virtual drawing routine for this primitive. All of the drawing operations
  /// are called on the draw_helper in order to abstract the graphics API from
  /// the primitive class
  virtual void DrawPrimitive( eLevelOfDetail lod, UWB_DrawHelper& draw_helper );

  /// Virtual function used to define the current attributes for drawing
  /// \param draw_helper this object is first passed to this function for setting
  ///        up it's attribute state and then the DrawPrimitive function is called
  ///        with this same object
  virtual void SetupDrawAttributes(UWB_DrawHelper& draw_helper);

protected:
  ///
  /// We can use the bounding box for our full geometry representation
  UWB_BoundingBox m_bounds;
  ///
  /// used for mouse handler
  vec3 m_mouse_down_point;
};
