// UW Bothell Graphics Library Copyright (c) 2006 University of Washington Bothell
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
/////////////////////////////////////////////////////////////////////////////////////////
#pragma once
#include "uwbgl_IWindowHandler.h"
#include "uwbgl_Common.h"
#include "uwbgl_Intersect2.h"
#include "uwbgl_DrawHelper2.h"

/// Common base class for graphics window handlers.
class UWB_WindowHandler : public UWB_IWindowHandler
{
public:
  UWB_WindowHandler();
  virtual ~UWB_WindowHandler();

  /// Subclasses should always call this when overriding this function. Checks that
  /// hParentWindow is an actual window and saves it to m_hAttachedWindow
  virtual bool InitializeHandler(HWND hAttachedWindow);

  /// Default implementation does nothing
  virtual void ShutDownHandler(){}

  /// Default mouse event handlers do nothing
  virtual void OnMouseButton(bool down, unsigned int nFlags, int hwX, int hwY){}
  virtual void OnMouseMove(unsigned int nFlags, int hwX, int hwY){}


  /// Transform point from hardware coordinates to device coordinates
  /// \param hardware_x, hardware_y [in]: hardware (or MFC window client) coordinate to transform
  /// \param device_x, device_y [out]: resulting device coordinate
  /// return true if successful
  bool HardwareToDevice(int hardwareX, int hardwareY, int& deviceX, int& deviceY) const;

  /// return aspect ratio of viewport (aspect ratio = width / height)
  float GetAspectRatio() const;

  /// Support for a world coordinate window
  virtual void SetWCWindow(const UWB_BoundingBox &window);
  virtual UWB_BoundingBox GetWCWindow() const;
  virtual void SetDeviceSize(int width, int height);
  virtual void GetDeviceSize(int &width, int &height) const;
  virtual void DeviceToWorld(int deviceX, int deviceY, float &wcX, float &wcY) const;
  virtual void WorldToDevice(float wcX, float wcY, int &deviceX, int &deviceY) const;
  virtual void LoadW2NDCXform() const = 0;
  virtual void DrawWCWindow(UWB_DrawHelper &drawHelper) const;
	
protected:
  /// Handle to window that this is connected to
  HWND m_hAttachedWindow;
  UWB_BoundingBox m_WCWindow; // Window for the World Coordinate
};
