// University of Washington Bothell Graphics Library
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////
#pragma once
#include "uwbgl_Common.h"
#include "uwbgl_DrawHelper2.h"

/// Transformation Class
/// This represents a translation, rotation, and scale that can be applied to a
/// graphics API specific transformation that will be applied to geometry before
/// it is drawn.
class UWB_XformInfo
{
public:
  UWB_XformInfo();

  /// Set up an API specific model stack with this transformation.
  /// \param draw_helper this class contains the API model stack routines
  void SetUpModelStack( UWB_DrawHelper& draw_helper ) const;

  /// Set rotation, translation and scale so they don't transform a point
  /// This has no effect on the pivot
  void ToIdentity();

  ///
  /// Get the translation vector component of the transformation
  vec3 GetTranslation() const;
  ///
  /// Set the translation vector component of the transformation
  void SetTranslation(vec3 translation);

  /// Get the scale component of the transformation. Scale is broken into
  /// scale in X, scale in Y, and scale in Z
  vec3 GetScale() const;
  /// Set the scale component of the transformation. Scale is broken into
  /// scale in X, scale in Y, and scale in Z
  void SetScale(vec3 scale);

  ///
  /// Get the rotation angle about the z-axis in radians
  float GetRotationInRadians() const;
  ///
  /// Get the rotation angle about the z-axis in degrees
  float GetRotationInDegrees() const;
  ///
  /// Set the rotation angle about the z-axis in radians
  void SetRotationRadians(float angle_in_radians);
  ///
  /// Set the rotation angle about the z-axis in degrees
  void SetRotationDegrees(float angle_in_degrees);

  ///
  /// Get the location about which rotation occurs
  vec3 GetPivot() const;
  ///
  /// Set the location about which rotation occurs
  void SetPivot(vec3 pivot);
  /// Draw the location about which rotation occurs.
  /// \param draw_helper The DrawPoint function of this class is used to
  ///        draw the pivot
  /// \param size the point size to use when drawing
  void DrawPivot(UWB_DrawHelper& draw_helper, float size=1.0f) const;

protected:
  vec3 m_translation;
  vec3 m_scale;
  float m_rotation_radians;
  vec3 m_pivot; //position that scale and rotation are centered about
};
